"""
This file holds specs that are in the new core3 archives but that don't have an entry
in the meta_data directory. This happens when the client itself collects the data
and puts it into the archive.
"""

from functools import partial

from insights.core.context import SerializedArchiveContext
from insights.core.spec_factory import simple_file
from insights.specs import Specs

simple_file = partial(simple_file, context=SerializedArchiveContext)


class Core3Specs(Specs):
    pass



import doctest

from insights.parsers import cron_log
from insights.tests import context_wrap

CRON_LOG = """
Feb 19 03:49:01 smtcfc0247 CROND[899752]: (root) CMD (/etc/run /etc/cron/every15Minutes /var/venus/logs/cron/every15Minutes 5 4.4 >/dev/null 2>&1)
Feb 19 03:49:01 smtcfc0247 CROND[899751]: (root) CMD (/etc/run /etc/cron/everyHour /var/venus/logs/cron/everyHour 5 65.8 >/dev/null 2>&1)
Feb 19 03:49:01 smtcfc0247 CROND[899753]: (root) CMD (/etc/run /etc/cron/every5Minutes /var/venus/logs/cron/every5Minutes 5 43.0 >/dev/null 2>&1)
Feb 19 03:49:01 smtcfc0247 CROND[908273]: (root) CMD (run-parts /etc/cron.hourly)
""".strip()


def test_doc_examples():
    env = {
        'cron_log': cron_log.CronLog(context_wrap(CRON_LOG))
    }
    failed, total = doctest.testmod(cron_log, globs=env)
    assert failed == 0


def test_cron_log():
    logs = cron_log.CronLog(context_wrap(CRON_LOG))
    etc_run_cmds = logs.get('/etc/run')
    assert 3 == len(etc_run_cmds)
    assert etc_run_cmds[0].get('timestamp') == "Feb 19 03:49:01"
    assert etc_run_cmds[1].get('message') == "(root) CMD (/etc/run /etc/cron/everyHour /var/venus/logs/cron/everyHour 5 65.8 >/dev/null 2>&1)"
    assert etc_run_cmds[2].get('procname') == "CROND[899753]"



"""
Foreman and Candlepin logs
==========================

Module for parsing the log files in foreman-debug archive

.. note::
    Please refer to its super-class :class:`insights.core.LogFileOutput` for
    usage information.

Parsers provided by this module:

CandlepinErrorLog - file ``sos_commands/foreman/foreman-debug/var/log/candlepin/error.log``
-------------------------------------------------------------------------------------------

CandlepinLog - file ``/var/log/candlepin/candlepin.log``
--------------------------------------------------------

ProductionLog - file ``/var/log/foreman/production.log``
--------------------------------------------------------

ProxyLog - file ``/var/log/foreman-proxy/proxy.log``
----------------------------------------------------

SatelliteLog - file ``/var/log/foreman-installer/satellite.log``
----------------------------------------------------------------

ForemanSSLAccessLog - file ``/var/log/httpd/foreman-ssl_access_ssl.log``
------------------------------------------------------------------------

ForemanSSLErrorLog - file ``/var/log/httpd/foreman-ssl_error_ssl.log``
----------------------------------------------------------------------

"""
from datetime import datetime

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.foreman_proxy_log)
class ProxyLog(LogFileOutput):
    """Class for parsing ``foreman-proxy/proxy.log`` file."""
    time_format = {
        'standard': '%d/%b/%Y:%H:%M:%S',  # 31/May/2016:09:57:34
        'error': '%Y-%m-%dT%H:%M:%S.%f'  # 2016-05-31T09:57:35.884636
    }


@parser(Specs.foreman_satellite_log)
class SatelliteLog(LogFileOutput):
    """Class for parsing ``foreman-installer/satellite.log`` file."""
    pass


@parser(Specs.foreman_production_log)
class ProductionLog(LogFileOutput):
    """Class for parsing ``foreman/production.log`` file.

    Each line is parsed into a dictionary with the following keys:

        * raw_message (str) - complete log line
        * message (str) - the body of the log
        * timestamp (datetime) - date and time of log as datetime object
    """
    def _parse_line(self, line):
        msg_info = {'raw_message': line}
        line_split = line.split(None, 2)
        if len(line_split) > 2:
            try:
                msg_info['timestamp'] = datetime.strptime(' '.join(line_split[:2]), self.time_format)
                msg_info['message'] = line_split[2]
            except ValueError:
                pass
        return msg_info


@parser(Specs.candlepin_log)
class CandlepinLog(LogFileOutput):
    """Class for parsing ``candlepin/candlepin.log`` file.

    Sample input::

        2016-09-09 13:45:52,650 [req=bd5a4284-d280-4fc5-a3d5-fc976b7aa5cc, org=] INFO org.candlepin.common.filter.LoggingFilter - Request: verb=GET, uri=/candlepin/consumers/f7677b4b-c470-4626-86a4-2fdf2546af4b
        2016-09-09 13:45:52,784 [req=bd5a4284-d280-4fc5-a3d5-fc976b7aa5cc, org=example_org] INFO  org.candlepin.common.filter.LoggingFilter - Response: status=200, content-type="application/json", time=134
        2016-09-09 13:45:52,947 [req=909ca4c5-f24e-4212-8f23-cc754d06ac57, org=] INFO org.candlepin.common.filter.LoggingFilter - Request: verb=GET, uri=/candlepin/consumers/f7677b4b-c470-4626-86a4-2fdf2546af4b/content_overrides
        2016-09-09 13:45:52,976 [req=909ca4c5-f24e-4212-8f23-cc754d06ac57, org=] INFO org.candlepin.common.filter.LoggingFilter - Response: status=200, content-type="application/json", time=29
        2016-09-09 13:45:53,072 [req=49becd26-5dfe-4d2f-8667-470519230d88, org=] INFO org.candlepin.common.filter.LoggingFilter - Request: verb=GET, uri=/candlepin/consumers/f7677b4b-c470-4626-86a4-2fdf2546af4b/release
        2016-09-09 13:45:53,115 [req=49becd26-5dfe-4d2f-8667-470519230d88, org=example_org] INFO  org.candlepin.common.filter.LoggingFilter - Response: status=200, content-type="application/json", time=43

    Each line is parsed into a dictionary with the following keys:

        * **raw_message(str)** - complete log line
        * **message(str)** - the body of the log
        * **timestamp(datetime)** - date and time of log as datetime object

    Examples:
        >>> cp_log_lines = cp_log.get('candlepin/consumers')
        >>> len(cp_log_lines)
        3
        >>> cp_log_lines[0].get('raw_message')
        '2016-09-09 13:45:52,650 [req=bd5a4284-d280-4fc5-a3d5-fc976b7aa5cc, org=] INFO org.candlepin.common.filter.LoggingFilter - Request: verb=GET, uri=/candlepin/consumers/f7677b4b-c470-4626-86a4-2fdf2546af4b'
        >>> cp_log_lines[0].get('message')
        '[req=bd5a4284-d280-4fc5-a3d5-fc976b7aa5cc, org=] INFO org.candlepin.common.filter.LoggingFilter - Request: verb=GET, uri=/candlepin/consumers/f7677b4b-c470-4626-86a4-2fdf2546af4b'
        >>> cp_log_lines[0].get('timestamp')
        datetime.datetime(2016, 9, 9, 13, 45, 52, 650000)
    """

    time_format = '%Y-%m-%d %H:%M:%S,%f'

    def _parse_line(self, line):
        # line format from /var/lib/tomcat/webapps/candlepin/WEB-INF/classes/logback.xml
        # <pattern>%d{ISO8601} [thread=%thread] [%X{requestType}=%X{requestUuid}, org=%X{org}, csid=%X{csid}] %-5p %c - %m%n</pattern>
        # http://logback.qos.ch/manual/layouts.html
        msg_info = {'raw_message': line}
        line_split = line.split(None, 2)
        if len(line_split) > 2:
            try:
                msg_info['timestamp'] = datetime.strptime(' '.join(line_split[:2]), self.time_format)
                msg_info['message'] = line_split[2]
            except ValueError:
                pass
        return msg_info


@parser(Specs.candlepin_error_log)
class CandlepinErrorLog(LogFileOutput):
    """
    Class for parsing ``candlepin/error.log`` file.

    Sample log contents::

        2016-09-07 13:56:49,001 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 14:07:33,735 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 14:09:55,173 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 15:20:33,796 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 15:27:34,367 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 16:49:24,650 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname
        2016-09-07 18:07:53,688 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.sync.Importer - Conflicts occurred during import that were not overridden:
        2016-09-07 18:07:53,690 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.sync.Importer - [DISTRIBUTOR_CONFLICT]
        2016-09-07 18:07:53,711 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.resource.OwnerResource - Recording import failure org.candlepin.sync.ImportConflictException: Owner has already imported from another subscription management application.

    Examples:
        >>> candlepin_log.get('req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28')[0]['raw_message']
        '2016-09-07 18:07:53,688 [req=d9dc3cfd-abf7-485e-b1eb-e1e28e4b0f28, org=org_ray] ERROR org.candlepin.sync.Importer - Conflicts occurred during import that were not overridden:'
        >>> from datetime import datetime
        >>> list(candlepin_log.get_after(datetime(2016, 9, 7, 16, 0, 0)))[0]['raw_message']
        '2016-09-07 16:49:24,650 [=, org=] WARN  org.apache.qpid.transport.network.security.ssl.SSLUtil - Exception received while trying to verify hostname'
    """
    pass


@parser(Specs.foreman_ssl_access_ssl_log)
class ForemanSSLAccessLog(LogFileOutput):
    """Class for parsing ``/var/log/httpd/foreman-ssl_access_ssl.log`` file.

    Sample log contents::

        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/385e688f-43ad-41b2-9fc7-593942ddec78 HTTP/1.1" 200 10736 "-" "-"
        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/status HTTP/1.1" 200 263 "-" "-"
        10.185.73.33 - 8a31cd915917666001591d6fb44602a7 [27/Mar/2017:13:34:52 -0400] "GET /pulp/repos/Acme_Inc/Library/RHEL7_Sat_Cap        sule_Servers/content/dist/rhel/server/7/7Server/x86_64/os/repodata/repomd.xml HTTP/1.1" 200 2018 "-" "urlgrabber/3.10 yum/3.4.3"
        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/4f8a39d0-38b6-4663-8b7e-03368be4d3ab/owner HTTP/1.1" 200 5159 "-"
        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/385e688f-43ad-41b2-9fc7-593942ddec78/compliance HTTP/1.1" 200 5527
        10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/4f8a39d0-38b6-4663-8b7e-03368be4d3ab HTTP/1.1" 200 10695 "-" "-"

    Each line is parsed into a dictionary with the following keys:

        * **raw_message(str)** - complete log line
        * **host(str)** - remote host's IP or hostname
        * **timestamp(datetime)** - date and time of http request, time zone is ignored

    Examples:
        >>> foreman_ssl_acess_log.get('consumers/385e688f-43ad-41b2-9fc7-593942ddec78')[0]['raw_message']
        '10.181.73.211 - rhcapkdc.example2.com [27/Mar/2017:13:34:52 -0400] "GET /rhsm/consumers/385e688f-43ad-41b2-9fc7-593942ddec78 HTTP/1.1" 200 10736 "-" "-"'
        >>> foreman_ssl_acess_log.get('consumers/385e688f-43ad-41b2-9fc7-593942ddec78')[0]['host']
        '10.181.73.211'
        >>> foreman_ssl_acess_log.get('consumers/385e688f-43ad-41b2-9fc7-593942ddec78')[0]['timestamp']
        datetime.datetime(2017, 3, 27, 13, 34, 52)
    """
    # Actual time format - '%d/%b/%Y:%H:%M:%S %z' but %z doesn't work for python 2
    time_format = '%d/%b/%Y:%H:%M:%S'

    def _parse_line(self, line):
        msg_info = {'raw_message': line}
        # Split enough to get datetime and host
        line_split = line.split(None, 4)
        if len(line_split) == 5:
            try:
                msg_info['timestamp'] = datetime.strptime(
                    line_split[3].strip('['), self.time_format
                )
                msg_info['host'] = line_split[0]
            except ValueError:
                pass
        return msg_info


@parser(Specs.foreman_ssl_error_ssl_log)
class ForemanSSLErrorLog(LogFileOutput):
    """
    Class for parsing ``/var/log/httpd/foreman-ssl_error_ssl.log`` file.

    .. note::

        Please refer to its super-class :class:`insights.core.LogFileOutput` for examples.

    Sample log contents::

        [Mon Aug 09 10:01:22.548717 2021] [ssl:warn] [pid 5881] [client 10.72.44.126:47190] AH02227: Failed to set r->user to 'SSL_CLIENT_S_DN_CN'
        [Mon Aug 09 11:02:23.229609 2021] [proxy_http:error] [pid 749] (20014)Internal error: [client 10.72.44.126:47920] AH01102: error reading status line from remote server yyy
        [Mon Aug 09 11:17:52.204503 2021] [proxy_http:error] [pid 5854] (20014)Internal error: [client 10.72.44.126:48016] AH01102: error reading status line from remote server yyy
    """

    time_format = '%a %b %d %H:%M:%S.%f %Y'



from insights.tests import context_wrap
from insights.parsers.systemid import SystemID

SYSTEMID = '''
<?xml version="1.0"?>
<params>
<param>
<value><struct>
<member>
<name>username</name>
<value><string>testuser</string></value>
</member>
<member>
<name>operating_system</name>
<value><string>redhat-release-workstation</string></value>
</member>
<member>
<name>description</name>
<value><string>Initial Registration Parameters: OS: redhat-release-workstation Release: 6Workstation CPU Arch: x86_64</string></value>
</member>
<member>
<name>checksum</name>
<value><string>b493da72be7cfb7e54c1d58c6aa140c9</string></value>
</member>
<member>
<name>profile_name</name>
<value><string>example_profile</string></value>
</member>
<member>
<name>system_id</name>
<value><string>ID-example</string></value>
</member>
<member>
<name>architecture</name>
<value><string>x86_64</string></value>
</member>
<member>
<name>os_release</name>
<value><string>6Workstation</string></value>
</member>
<member>
<name>fields</name>
<value><array><data>
<value><string>system_id</string></value>
<value><string>os_release</string></value>
<value><string>operating_system</string></value>
<value><string>architecture</string></value>
<value><string>username</string></value>
<value><string>type</string></value>
</data></array></value>
</member>
<member>
<name>type</name>
<value><string>REAL</string></value>
</member>
</struct></value>
</param>
</params>
'''.strip()


def test_systemid():
    info = SystemID(context_wrap(SYSTEMID, path='/etc/sysconfig/rhn/systemid'))

    assert info.get("username") == 'testuser'
    assert info.get("operating_system") == 'redhat-release-workstation'
    assert info.get("description") == 'Initial Registration Parameters: OS: redhat-release-workstation Release: 6Workstation CPU Arch: x86_64'
    assert info.get("checksum") == 'b493da72be7cfb7e54c1d58c6aa140c9'
    assert info.get("profile_name") == 'example_profile'
    assert info.get("system_id") == 'ID-example'
    assert info.get("architecture") == 'x86_64'
    assert info.get("os_release") == '6Workstation'
    assert info.get("type") == 'REAL'

    assert info.file_name == 'systemid'
    assert info.file_path == '/etc/sysconfig/rhn/systemid'



"""
CpuInfo - file ``/proc/cpuinfo``
================================

This parser reads the content of the ``/proc/cpuinfo`` file and parses it
into a dictionary of lists, keyed on the left hand column of the cpuinfo
output.

The object also provides properties for the standard information about the
CPU and motherboard architecture.

Sample input::

    processor       : 0
    vendor_id       : GenuineIntel
    cpu family      : 6
    model           : 45
    model name      : Intel(R) Xeon(R) CPU E5-2690 0 @ 2.90GHz
    stepping        : 2
    microcode       : 1808
    cpu MHz         : 2900.000
    cache size      : 20480 KB
    physical id     : 0
    siblings        : 1
    core id         : 0
    cpu cores       : 1
    apicid          : 0
    flags           : fpu vme de pse tsc msr pae mce
    address sizes   : 40 bits physical, 48 bits virtual
    bugs            : cpu_meltdown spectre_v1 spectre_v2 spec_store_bypass l1tf mds swapgs taa itlb_multihit

    processor       : 1
    vendor_id       : GenuineIntel
    cpu family      : 6
    model           : 45
    model name      : Intel(R) Xeon(R) CPU E5-2690 0 @ 2.90GHz
    stepping        : 2
    microcode       : 1808
    cpu MHz         : 2900.000
    cache size      : 20480 KB
    physical id     : 2
    siblings        : 1
    core id         : 0
    cpu cores       : 1
    apicid          : 2
    flags           : fpu vme de pse tsc msr pae mce
    address sizes   : 40 bits physical, 48 bits virtual
    bugs            : cpu_meltdown spectre_v1 spectre_v2 spec_store_bypass l1tf mds swapgs taa itlb_multihit

Examples:

    >>> cpu_info.cpu_count
    2
    >>> sorted(cpu_info.apicid)
    ['0', '2']
    >>> cpu_info.socket_count
    2
    >>> cpu_info.vendor
    'GenuineIntel'
    >>> "fpu" in cpu_info.flags
    True
    >>> cpu_info.model_name
    'Intel(R) Xeon(R) CPU E5-2690 0 @ 2.90GHz'
    >>> cpu_info.get_processor_by_index(0)['cpus']
    '0'
    >>> cpu_info.get_processor_by_index(0)['vendors']
    'GenuineIntel'
    >>> cpu_info.microcode
    '1808'
"""
from collections import defaultdict
from insights import Parser, parser, defaults, get_active_lines, LegacyItemAccess
from insights.specs import Specs


@parser(Specs.cpuinfo)
class CpuInfo(LegacyItemAccess, Parser):
    """
    CpuInfo parser - able to be used as a dictionary through the
    LegacyItemAccess mixin class.

    The following items are remapped into lists, with the element number
    corresponding to the CPU.  For example, given the following input::

        processor       : 1
        vendor_id       : GenuineIntel
        cpu family      : 6
        model           : 45
        model name      : Intel(R) Xeon(R) CPU E5-2690 0 @ 2.90GHz
        stepping        : 2
        microcode       : 1808
        cpu MHz         : 2900.000
        cache size      : 20480 KB
        physical id     : 2
        siblings        : 1
        core id         : 0
        cpu cores       : 1
        apicid          : 2
        address sizes   : 40 bits physical, 48 bits virtual
        bugs            : cpu_meltdown spectre_v1 spectre_v2 spec_store_bypass l1tf mds swapgs taa itlb_multihit

    The following keys would be lists of:

    * **cpus** - the *processor* line (e.g. ``1``)
    * **sockets** - the *physical id* line (e.g. ``2``)
    * **vendors** - the *vendor_id* line (e.g. ``GenuineIntel``)
    * **models** - the *model name* line (e.g. ``Intel(R) Xeon(R) CPU E5-2690 0 @ 2.90GHz``)
    * **model_ids** - the *model* line (e.g. ``45``)
    * **families** - the *cpu family* line (e.g. ``6``)
    * **clockspeeds** - the *cpu MHz* line (e.g. ``2900.000``)
    * **cache_sizes** - the *cache size* line (e.g. ``20480 KB``)
    * **cpu_cores** - the *cpu cores* line (e.g. ``1``)
    * **apicid** - the *apicid* line (e.g. ``1``)
    * **stepping** - the *stepping* line (e.g. ``2``)
    * **address_sizes** - the *address sizes* line (e.g. ``40 bits physical, 48 bits virtual``)
    * **bugs** - the *bugs* line (e.g. ``cpu_meltdown spectre_v1 spectre_v2 spec_store_bypass l1tf mds swapgs taa itlb_multihit``)
    """

    def parse_content(self, content):

        self.data = defaultdict(list)
        mappings = {
            "processor": "cpus",
            "physical id": "sockets",
            "vendor_id": "vendors",
            "model name": "models",
            "model": "model_ids",
            "cpu family": "families",
            "cpu MHz": "clockspeeds",
            "cache size": "cache_sizes",
            "cpu cores": "cpu_cores",
            "apicid": "apicid",
            "flags": "flags",
            "stepping": "stepping",
            "Features": "features",
            "CPU implementer": "cpu_implementer",
            "CPU architecture": "cpu_architecture",
            "CPU variant": "cpu_variant",
            "CPU part": "cpu_part",
            "CPU revision": "cpu_revision",
            "cpu": "cpu",
            "revision": "revision",
            "address sizes": "address_sizes",
            "bugs": "bugs",
            "microcode": "microcode",
            "cpu MHz static": "clockspeeds",
            "features": "features"
        }

        for line in get_active_lines(content, comment_char="COMMAND>"):
            key, value = [p.strip() for p in line.split(":", 1)]
            # For s390x the : symbol is after the number instead of before.
            # So re-split and set the key and value before checking mappings.
            if key.startswith("processor") and key[-1].isdigit():
                key, value = key.split(" ", 1)

            if key in mappings:
                self.data[mappings[key]].append(value)

        if "cpu" in self.data and "POWER" in self.data["cpu"][0]:
            # this works differently than on x86 and is not per-cpu
            model_id = self.data["model_ids"][0]
            cpu_cnt = self.cpu_count
            self.data["model_ids"] = [model_id] * cpu_cnt

        # s390x cpuinfo is setup drastically differently than other arches.
        # It doesn't print the same information for each cpu, like other arches.
        # So any info not repeated per cpu, copy, delete and then add for each cpu entry.
        if "vendors" in self.data and "IBM/S390" in self.data["vendors"][0]:
            vendor = self.data["vendors"][0]
            features = self.data["features"][0]
            cpu_cnt = self.cpu_count
            self.data["vendors"] = [vendor] * cpu_cnt
            self.data["features"] = [features] * cpu_cnt

        self.data = dict(self.data)

    def __iter__(self):
        """
        Iterating through this object will yield the ``get_processor_by_index``
        information for each CPU.
        """
        for idx in range(len(self["cpus"])):
            yield self.get_processor_by_index(idx)

    @property
    @defaults()
    def cpu_speed(self):
        """
        str: Returns the CPU speed of the first CPU.
        """
        return self.data["clockspeeds"][0]

    @property
    @defaults()
    def cache_size(self):
        """
        str: Returns the cache size of the first CPU.
        """
        return self.data["cache_sizes"][0]

    @property
    def cpu_count(self):
        """
        int : Returns the number of CPUs.
        """
        return len(self.data.get("cpus", []))

    @property
    def apicid(self):
        """
        list: Returns the list of apicid of the processor.
        """
        return self.data.get("apicid", [])

    @property
    def socket_count(self):
        """
        int: Returns the number of sockets.  This is distinct from the number
        of CPUs.
        """
        return len(set(self.data.get("sockets", [])))

    @property
    @defaults()
    def model_name(self):
        """
        str: Returns the model name of the first CPU.
        """
        return self.data["models"][0]

    @property
    @defaults()
    def model_number(self):
        """
        str: Returns the model ID of the first CPU.
        """
        return self.data["model_ids"][0]

    @property
    @defaults([])
    def flags(self):
        """
        list: Returns a list of feature flags for the first CPU.
        """
        return self.data["flags"][0].split()

    @property
    @defaults()
    def vendor(self):
        """
        str: Returns the vendor of the first CPU.
        """
        return self.data["vendors"][0]

    @property
    @defaults()
    def microcode(self):
        """
        str: Returns the microcode of the first CPU.
        """
        return self.data["microcode"][0]

    def get_processor_by_index(self, index):
        """
        Construct a dictionary of the information stored for the given CPU.

        Parameters:
            index (int): The CPU index to retrieve.

        Returns:
            dict: A dictionary of the information for that CPU.
        """
        return dict((k, v[index]) for k, v in self.data.items())



from insights.tests import context_wrap
from insights.parsers.dmesg import DmesgLineList
from insights.parsers.dmesg_log import DmesgLog
from insights.combiners.dmesg import Dmesg

DMESG_LOG = """
[    0.000000] Initializing cgroup subsys cpu
[    0.000000] Linux version 3.10.0-862.el7.x86_64 (mockbuild@x86-034.build.eng.bos.redhat.com) \
(gcc version 4.8.5 20150623 (Red Hat 4.8.5-28) (GCC) ) #1 SMP Wed Mar 21 18:14:51 EDT 2018
[    2.090905] SELinux:  Completing initialization.
[    2.090907] SELinux:  Setting up existing superblocks.
[    2.099684] systemd[1]: Successfully loaded SELinux policy in 82.788ms.
[    2.117410] ip_tables: (C) 2000-2006 Netfilter Core Team
[    2.117429] systemd[1]: Inserted module 'ip_tables'
[    2.376551] systemd-journald[441]: Received request to flush runtime journal from PID 1
[    2.716874] cryptd: max_cpu_qlen set to 100
[    2.804152] AES CTR mode by8 optimization enabled
[    2.874051] XFS (vda1): Mounting V5 Filesystem
[    2.919331] alg: No test for __gcm-aes-aesni (__driver-gcm-aes-aesni)
[    2.939496] [TTM] Zone  kernel: Available graphics memory: 507714 kiB
[    2.939498] [TTM] Initializing pool allocator
[    2.939502] [TTM] Initializing DMA pool allocator
""".strip()

DMESG_CMD = """
[    2.939498] [TTM] Initializing pool allocator
[    2.939502] [TTM] Initializing DMA pool allocator
[    2.940800] [drm] fb mappable at 0xFC000000
[    2.940947] fbcon: cirrusdrmfb (fb0) is primary device
[    2.957375] Console: switching to colour frame buffer device 128x48
[    2.959322] cirrus 0000:00:02.0: fb0: cirrusdrmfb frame buffer device
[    2.959334] [drm] Initialized cirrus 1.0.0 20110418 for 0000:00:02.0 on minor 0
[    3.062459] XFS (vda1): Ending clean mount
[    5.048484] ip6_tables: (C) 2000-2006 Netfilter Core Team
[    5.102434] Ebtables v2.0 registered
[    5.359444] IPv6: ADDRCONF(NETDEV_UP): eth0: link is not ready
[    5.406243] nf_conntrack version 0.5.0 (7933 buckets, 31732 max)
[    5.673082] Netfilter messages via NETLINK v0.30.
[    5.678631] ip_set: protocol 6
[   23.936112] random: crng init done
""".strip()


def test_dmesg_cmd():
    dmesg_cmd = DmesgLineList(context_wrap(DMESG_CMD))
    dmesg = Dmesg(dmesg_cmd, None)
    assert dmesg.dmesg_log_available is False
    assert hasattr(dmesg, "dmesg_log") is False
    assert dmesg.dmesg_cmd_available is True
    assert dmesg.dmesg_cmd_wrapped is True
    assert dmesg.dmesg_cmd is not False


def test_dmesg_log():
    dmesg_log = DmesgLog(context_wrap(DMESG_LOG))
    dmesg = Dmesg(None, dmesg_log)
    assert dmesg.dmesg_cmd_available is False
    assert hasattr(dmesg, "dmesg_cmd") is False
    assert dmesg.dmesg_log_available is True
    assert dmesg.dmesg_log_wrapped is False
    assert dmesg.dmesg_log is not False


def test_combined():
    dmesg_cmd = DmesgLineList(context_wrap(DMESG_CMD))
    dmesg_log = DmesgLog(context_wrap(DMESG_LOG))
    dmesg = Dmesg(dmesg_cmd, dmesg_log)
    assert dmesg.dmesg_cmd_available is True
    assert dmesg.dmesg_log_available is True


def test_empty():
    dmesg = Dmesg(None, None)
    assert dmesg.dmesg_cmd_available is False
    assert dmesg.dmesg_log_available is False



from insights.parsers.pcs_config import PCSConfig
from insights.parsers import pcs_config
from insights.tests import context_wrap
import doctest

NORMAL_PCS_CONFIG = """
Cluster Name: cluster-1
Corosync Nodes:
 node-1 node-2
Pacemaker Nodes:
 node-1 node-2

Resources:
 Clone: clone-1
  Meta Attrs: interleave=true ordered=true
  Resource: res-1 (class=ocf provider=pacemaker type=controld)
   Operations: start interval=0s timeout=90 (dlm-start-interval-0s)
               stop interval=0s timeout=100 (dlm-stop-interval-0s)
               monitor interval=30s on-fail=fence (dlm-monitor-interval-30s)
 Clone: clone-2
  Meta Attrs: interleave=true ordered=true
  Resource: res-2 (class=ocf provider=pacemaker type=controld)
   Operations: start interval=0s timeout=90 (dlm-start-interval-0s)
               stop interval=0s timeout=100 (dlm-stop-interval-0s)
               monitor interval=30s on-fail=fence (dlm-monitor-interval-30s)
 Group: grp-1
  Resource: res-1 (class=ocf provider=heartbeat type=IPaddr2)
   Attributes: ip=10.0.0.1 cidr_netmask=32
   Operations: monitor interval=120s (ip_monitor-interval-120s)
               start interval=0s timeout=20s (ip_-start-interval-0s)
               stop interval=0s timeout=20s (ip_-stop-interval-0s)
  Resource: res-2 (class=ocf provider=heartbeat type=Filesystem)
   Attributes: device=/dev/lv_exzpr directory= fstype=xfs run_fsck=yes fast_stop=yes
   Operations: start interval=0s timeout=60 (fs_exzpr-start-interval-0s)
               stop interval=0s timeout=60 (fs_exzpr-stop-interval-0s)
               monitor interval=30s timeout=60 (fs_exzpr-monitor-interval-30s)

Stonith Devices:
Fencing Levels:

Location Constraints:
  Resource: fence-1
    Disabled on: res-mgt (score:-INFINITY) (id:location-fence-1--INFINITY)
  Resource: res-1
    Enabled on: res-mcast (score:INFINITY) (role: Started) (id:cli-prefer-res)
Ordering Constraints:
  start clone-1 then start clone-x (kind:Mandatory) (id:clone-mandatory)
  start clone-2 then start clone-y (kind:Mandatory) (id:clone-mandatory)
Colocation Constraints:
  clone-1 with clone-x (score:INFINITY) (id:clone-INFINITY)
  clone-2 with clone-x (score:INFINITY) (id:clone-INFINITY)

Resources Defaults:
 resource-stickiness: 100
 migration-threshold: 3
Operations Defaults:
 No defaults set

Cluster Properties:
 cluster-infrastructure: corosync
 cluster-name: cluster-1
 dc-version: 1.1.13-10.el7_2.4-44eb2dd
 have-watchdog: false
 no-quorum-policy: ignore
 stonith-enable: true
 stonith-enabled: false
""".strip()

PCS_CONFIG_ON_REMOTE = """
Cluster Name: test_cluster
Corosync Nodes:
Pacemaker Nodes:
 node1 node2 node3

Resources:
 Resource: dummy1 (class=ocf provider=heartbeat type=Dummy)
  Operations: migrate_from interval=0s timeout=20s (dummy1-migrate_from-interval-0s)
              migrate_to interval=0s timeout=20s (dummy1-migrate_to-interval-0s)
              monitor interval=10s timeout=20s (dummy1-monitor-interval-10s)
              reload interval=0s timeout=20s (dummy1-reload-interval-0s)
              start interval=0s timeout=20s (dummy1-start-interval-0s)
              stop interval=0s timeout=20s (dummy1-stop-interval-0s)
 Resource: remote1 (class=ocf provider=pacemaker type=remote)
  Attributes: server=remote1
  Operations: migrate_from interval=0s timeout=60s (remote1-migrate_from-interval-0s)
              migrate_to interval=0s timeout=60s (remote1-migrate_to-interval-0s)
              monitor interval=60s timeout=30s (remote1-monitor-interval-60s)
              reload interval=0s timeout=60s (remote1-reload-interval-0s)
              start interval=0s timeout=60s (remote1-start-interval-0s)
              stop interval=0s timeout=60s (remote1-stop-interval-0s)

Stonith Devices:
Fencing Levels:

Location Constraints:
Ordering Constraints:
Colocation Constraints:
Ticket Constraints:

Alerts:
 No alerts defined

Resources Defaults:
 No defaults set
Operations Defaults:
 No defaults set

Cluster Properties:
 cluster-infrastructure: corosync
 cluster-name: my_cluster
 dc-version: 2.0.3-5.el8-4b1f869f0f
 have-watchdog: false
 last-lrm-refresh: 1667289123
""".strip()


def test_pcs_config_basic():
    pcs = PCSConfig(context_wrap(NORMAL_PCS_CONFIG))
    assert pcs.get("Cluster Name") == "cluster-1"
    assert pcs.get("Corosync Nodes") == ["node-1", "node-2"]
    assert pcs.get("Pacemaker Nodes") == ["node-1", "node-2"]
    assert pcs.get("Unknown Key") is None


def test_pcs_config_normal():
    pcs = PCSConfig(context_wrap(NORMAL_PCS_CONFIG))
    assert "cluster-infrastructure: corosync" in pcs.get("Cluster Properties")
    assert "stonith-enabled: false" in pcs.get("Cluster Properties")
    assert pcs.get("Resources Defaults") == ["resource-stickiness: 100", "migration-threshold: 3"]
    assert "cluster-infrastructure" in pcs.cluster_properties
    assert "have-watchdog" in pcs.cluster_properties
    assert pcs.cluster_properties.get("have-watchdog") == "false"


def test_pcs_config_on_remote():
    pcs = PCSConfig(context_wrap(PCS_CONFIG_ON_REMOTE))
    assert pcs.get("Corosync Nodes") == []


def test_pcs_config_documentation():
    env = {
        'pcs_config': PCSConfig(context_wrap(NORMAL_PCS_CONFIG)),
    }
    failed, total = doctest.testmod(pcs_config, globs=env)
    assert failed == 0



from pytest import mark

from insights.cleaner import Cleaner
from insights.cleaner.keyword import Keyword
from insights.client.config import InsightsConfig

test_data = 'test\nabcd\n \n\n1234\npwd: p4ssw0rd\n'.splitlines()


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_keyword_empty_not_change(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {})  # empty keywords
    ret = pp.clean_content(test_data, [])
    # content is NOT changed
    assert test_data == ret


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_keyword_changed_keyword(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {'keywords': ['test']})
    ret = pp.clean_content(test_data, [])
    # content IS changed
    assert test_data != ret
    assert 'test' not in ret[0]
    assert 'keyword0' in ret[0]
    assert ret[1] == test_data[1]


@mark.parametrize("obfuscate", [True, False])
def test_clean_content_keyword_no_such_keyword_to_change(obfuscate):
    conf = InsightsConfig(obfuscate=obfuscate)

    pp = Cleaner(conf, {'keywords': ['t_e_s_t']})  # no such keyword
    ret = pp.clean_content(test_data, [])
    # content is NOT changed
    assert test_data == ret
    assert ret[1] == test_data[1]


def test_keyword_empty():
    kw = Keyword([])  # no keyword
    assert kw._kw_db == dict()



import copy
import os
import six
import tempfile

from glob import glob
from logging import getLogger
from re import findall
from sys import exit

from insights.client.connection import InsightsConnection
from insights.client.constants import InsightsConstants as constants
from insights.util.subproc import call


NONCOMPLIANT_STATUS = 2
OUT_OF_MEMORY_STATUS = -9  # 247
COMPLIANCE_CONTENT_TYPE = 'application/vnd.redhat.compliance.something+tgz'
POLICY_FILE_LOCATION = '/usr/share/xml/scap/ssg/content/'
SCAP_DATASTREAMS_PATH = '/usr/share/xml/scap/'
SSG_PACKAGE = 'scap-security-guide'
REQUIRED_PACKAGES = [SSG_PACKAGE, 'openscap-scanner', 'openscap']
OOM_ERROR_LINK = 'https://access.redhat.com/articles/6999111'

# SSG versions that need the <version> in XML repaired
VERSIONS_FOR_REPAIR = '0.1.18 0.1.19 0.1.21 0.1.25'.split()
SNIPPET_TO_FIX = '<version>0.9</version>'

logger = getLogger(__name__)


class ComplianceClient:
    def __init__(self, os_version=None, ssg_version=None, config=None):
        self._inventory_id = None
        self.os_major, self.os_minor = os_version if os_version else [None, None]
        self.ssg_version = ssg_version
        config = copy.deepcopy(config)
        # Do nothing when it's already `legacy_upload=False`
        if config.legacy_upload:
            # use legacy_upload=False basic configurations by force
            config.legacy_upload = False
            config.base_url = constants.base_url
        self.conn = InsightsConnection(config)

    def fetch_tailoring_content(self, policy):
        if 'os_minor_version' in policy and policy['os_minor_version'] != self.os_minor:
            return None

        logger.debug(
            "Checking if policy {0} is a tailored policy. Starting tailoring file download...".format(
                policy['ref_id']
            )
        )
        response = self.conn.session.get(
            "{0}/compliance/v2/policies/{1}/tailorings/{2}/tailoring_file".format(
                self.conn.base_url, policy['id'], self.os_minor
            )
        )
        logger.debug("Response code: {0}".format(response.status_code))

        if response.status_code == 204:
            logger.debug(
                "Policy {0} is not tailored, continuing with default rule and value selections...".format(
                    policy['ref_id']
                )
            )
            return None

        if not response.ok:
            logger.debug(
                "Something went wrong during downloading the tailoring file of {0}. The expected status code is 200, got {1}".format(
                    policy['ref_id'], response.status_code
                )
            )
            return None

        if response.content is None or response.headers['Content-Type'] != "application/xml":
            logger.debug(response.content)
            logger.debug(response.return_value)
            logger.debug(
                "Problem with the content of the downloaded tailoring file of {0}. The expected format is xml, got {1}".format(
                    policy['ref_id'], response.headers['Content-Type']
                )
            )
            return None

        return response.content

    def download_tailoring_file(self, policy):
        # Download tailoring file to pass as argument to run_scan
        content = self.fetch_tailoring_content(policy)
        # Check if the content is empty, and if so, don't create the file at all
        if content is None or not content.strip():
            logger.debug(
                "The tailoring file for policy {0} is empty. Skipping file creation.".format(
                    policy['ref_id']
                )
            )
            return None

        # Create a temporary file if content is valid (not empty)
        tailoring_file_path = tempfile.mkstemp(
            prefix='oscap_tailoring_file-{0}.'.format(policy['ref_id']),
            suffix='.xml',
            dir='/var/tmp',
        )[1]

        with open(tailoring_file_path, mode="w+b") as f:
            f.write(content)
            logger.info(
                "Saved tailoring file for {0} to {1}".format(policy['ref_id'], tailoring_file_path)
            )

        logger.debug("Policy {0} tailoring file download finished".format(policy['ref_id']))

        return tailoring_file_path

    def profile_files(self):
        return glob("{0}*rhel{1}-ds.xml".format(POLICY_FILE_LOCATION, self.os_major))

    def find_scap_policy(self, profile_ref_id):
        grepcmd = 'grep -H ' + profile_ref_id + ' ' + ' '.join(self.profile_files())
        if not six.PY3:
            grepcmd = grepcmd.encode()
        rc, grep = call(grepcmd, keep_rc=True)
        if rc:
            logger.error(
                'XML profile file not found matching ref_id {0}\n{1}\n'.format(profile_ref_id, grep)
            )
            return None
        filenames = findall(SCAP_DATASTREAMS_PATH + '.+xml', grep)
        if not filenames:
            logger.error(
                'No XML profile files found matching ref_id {0}\n{1}\n'.format(
                    profile_ref_id, ' '.join(filenames)
                )
            )
            exit(constants.sig_kill_bad)
        return filenames[0]

    def build_oscap_command(self, profile_ref_id, policy_xml, output_path, tailoring_file_path):
        command = 'oscap xccdf eval --profile ' + profile_ref_id
        if tailoring_file_path:
            command += ' --tailoring-file ' + tailoring_file_path
        command += ' --results ' + output_path + ' ' + policy_xml
        return command

    def run_scan(self, profile_ref_id, policy_xml, output_path, tailoring_file_path=None):
        if policy_xml is None:
            return
        logger.info('Running scan for {0}... this may take a while'.format(profile_ref_id))
        env = os.environ.copy()
        env.update({'TZ': 'UTC'})
        oscap_command = self.build_oscap_command(
            profile_ref_id, policy_xml, output_path, tailoring_file_path
        )
        if not six.PY3:
            oscap_command = oscap_command.encode()
        rc, oscap = call(oscap_command, keep_rc=True, env=env)

        if rc and rc == OUT_OF_MEMORY_STATUS:
            logger.error('Scan failed due to insufficient memory')
            logger.error('More information can be found here: {0}'.format(OOM_ERROR_LINK))
            exit(constants.sig_kill_bad)

        if rc and rc != NONCOMPLIANT_STATUS:
            logger.error('Scan failed')
            logger.error(rc)
            logger.error(oscap)
            exit(constants.sig_kill_bad)

    # Helper function that traverses through the XCCDF report and replaces the content of each
    # matching xpath with an empty string
    def obfuscate(self, tree, xpaths):
        for xpath in xpaths:
            for node in tree.findall(xpath):
                node.text = ''

    def results_need_repair(self):
        return self.ssg_version in VERSIONS_FOR_REPAIR

    def repair_results(self, results_file, content):
        if not self.ssg_version:
            logger.warning("Couldn't repair SSG version in results file %s", results_file)
            return

        replacement = '<version>{0}</version>'.format(self.ssg_version)
        is_repaired = False
        for idx, line in enumerate(content):
            if not (is_repaired or SNIPPET_TO_FIX not in line):
                content[idx] = line.replace(SNIPPET_TO_FIX, replacement)
                is_repaired = True
                logger.debug(
                    'Substituted "%s" with "%s" in %s', SNIPPET_TO_FIX, replacement, results_file
                )

        if is_repaired:
            logger.debug('Repaired version in results file %s', results_file)

        return content

    @property
    def inventory_id(self):
        if self._inventory_id:
            return self._inventory_id

        system = self.conn._fetch_system_by_machine_id()
        if isinstance(system, dict) and 'id' in system:
            self._inventory_id = system.get('id')
            return self._inventory_id
        else:
            logger.error('Failed to find system in Inventory')
            exit(constants.sig_kill_bad)

    def assignable_policies(self):
        url = "{0}/compliance/v2/policies?filter=(os_major_version={1} and os_minor_version={2})&limit=100"
        full_url = url.format(self.conn.base_url, self.os_major, self.os_minor)
        logger.debug("Fetching policies with: {0}".format(full_url))
        response = self.conn.session.get(full_url)
        logger.debug("Content of the response {0} - {1}".format(response, response.content))
        assigned_policies = [item["id"] for item in self.get_system_policies()]

        if response.status_code == 200:
            policies = response.json().get('data', [])
            if not policies:
                logger.warning(
                    "System is not assignable to any policy. Create supported policy using the Compliance web UI.\n"
                )
                return constants.sig_kill_bad
            else:
                print("Assigned     ID" + " " * 39 + "Title")
                for policy in policies:
                    is_assigned = policy['id'] in assigned_policies
                    print("%-12s %-40s %s" % (is_assigned, policy['id'], policy['title']))
            return 0
        else:
            logger.error("An error has occurred while communicating with the API.\n")
            return constants.sig_kill_bad

    def policy_link(self, policy_id, opt):
        url = "{0}/compliance/v2/policies/{1}/systems/{2}"
        full_url = url.format(self.conn.base_url, policy_id, self.inventory_id)
        logger.debug("Fetching: {0}".format(full_url))
        response = getattr(self.conn.session, opt)(full_url)
        logger.debug("Content of the response {0} - {1}".format(response, response.content))

        operation = "unassigned" if opt == "delete" else "assigned"
        if response.status_code == 202:
            logger.info("Successfully {0} policy (ID {1}).\n".format(operation, policy_id))
            return 0
        else:
            logger.error(
                "Policy ID {0} can not be {1}. "
                "Refer to the /var/log/insights-client/insights-client.log for more details.".format(
                    policy_id,
                    operation,
                )
            )
            return constants.sig_kill_bad

    def get_system_policies(self):
        url = "{0}/compliance/v2/systems/{1}/policies".format(self.conn.base_url, self.inventory_id)
        logger.debug("Fetching policies with: {0}".format(url))
        response = self.conn.session.get(url)
        logger.debug("Content of the response {0} - {1}".format(response, response.content))

        if response.status_code == 200:
            return response.json().get('data', [])
        else:
            return []



# -*- coding: UTF-8 -*-

from insights.client.collection_rules import InsightsUploadConf
from insights.client.config import InsightsConfig


def insights_upload_conf(*args, **kwargs):
    """
    Instantiates InsightsUploadConf with a config created with given arguments.
    """
    config = InsightsConfig(*args, **kwargs)
    return InsightsUploadConf(config)



"""
Disk Usage parsers
==================

Module for the processing of output from the ``du`` command.

Parsers provided by this module include:

DiskUsageDir - command ``du -s -k {directory}``
-----------------------------------------------

"""
from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


class DiskUsage(CommandParser, dict):
    """
    Reads output of du command and turns it into a dictionary with pathname as
    key and size in blocks.

    Supports parsing input data as long as output is 2 column data with first
    column as space size with integer values only and second as pathname which
    can be a file or directory. Space size with decimal values or unit suffixes
    like M, GiB is not supported.

    du command produces output in 1K blocks unless block size is specified in
    command options or an environment variable. This parser is intended to be
    used only with default block size of 1K which is also equal to plain "du"
    or "du -k".

    Sample input data::

        56	/var/lib/alternatives
        4	/var/lib/logrotate
        5492	/var/lib/mlocate
        20	/var/lib/NetworkManager
        186484	/var/lib/pgsql
        856	/var/lib/rhsm
        110712	/var/lib/rpm
        4	/var/lib/rsyslog
        64	/var/lib/systemd
        15200	/var/lib/yum

    Examples:
        >>> '/var/lib/pgsql' in disk_usage
        True
        >>> disk_usage.get('/var/lib/pgsql')
        186484
        >>> int(disk_usage.get('/var/lib/pgsql') / 1024) # to MiB
        182

    Raises:
        SkipComponent: When no data could be parsed.
        ParseException: Raised when any problem parsing the command output.
    """

    def parse_content(self, content):
        """
        Parse input data into a dictionary.
        """
        # For errors like :
        # /bin/du: cannot read directory '/somepath'
        # /bin/du: cannot access `/somepath': No such file or directory
        du_error = 'bin/du: '

        for line in content:
            if du_error in line:
                continue

            line_split = line.split(None, 1)
            if len(line_split) != 2:
                raise ParseException("Could not parse line: {0}".format(line))
            size, path = line_split
            path = path.rstrip()
            if path.startswith('.'):
                raise ParseException("Relative paths not supported: {0}'".
                                     format(line))
            if path and size.isdigit():
                self[path] = int(size)
            else:
                raise ParseException("Could not parse line: '{0}'".
                                     format(line))
        if len(self) == 0:
            raise SkipComponent('No data parsed')


@parser(Specs.du_dirs)
class DiskUsageDir(DiskUsage):
    """
    Parser class for processing du output for multiple directories, each
    collected using ``du -s -k {directory}``.
    """
    pass



"""
The filters module allows developers to apply filters to datasources,
by adding them directly or through dependent components like parsers
and combiners. A filter is a simple string, and it matches if it is contained
anywhere within a line.

If a datasource has filters defined, it will return only lines matching at
least one of them. If a datasource has no filters, it will return all lines.

Filters can be added to components like parsers and combiners, to apply consistent
filtering to multiple underlying datasources that are configured as filterable.

Filters aren't applicable to "raw" datasources, which are created with
``kind=RawFileProvider`` and have ``RegistryPoint`` instances with ``raw=True``.

The addition of a single filter can cause a datasource to change from returning
all lines to returning just those that match. Therefore, any filtered
datasource should have at least one filter in the commit introducing it so
downstream components don't inadvertently change its behavior.

The benefit of this fragility is the ability to drastically reduce in-memory
footprint and archive sizes. An additional benefit is the ability to evaluate
only lines known to be free of sensitive information.

Filters added to a ``RegistryPoint`` will be applied to all datasources that
implement it. Filters added to a datasource implementation apply only to that
implementation.

For example, a filter added to ``Specs.ps_auxww`` will apply to
``DefaultSpecs.ps_auxww``, ``InsightsArchiveSpecs.ps_auxww``,
``SosSpecs.ps_auxww``, etc. But a filter added to `DefaultSpecs.ps_auxww` will
only apply to ``DefaultSpecs.ps_auxww``. See the modules in ``insights.specs``
for those classes.

Filtering can be disabled globally by setting the environment variable
``INSIGHTS_FILTERS_ENABLED=False``. This means that no datasources will be
filtered even if filters are defined for them.
"""

import os
import pkgutil
import six
import yaml as ser
from collections import defaultdict

import insights
from insights.core import dr, plugins
from insights.util import parse_bool

_CACHE = {}
FILTERS = defaultdict(dict)
ENABLED = parse_bool(os.environ.get("INSIGHTS_FILTERS_ENABLED"), default=True)
MAX_MATCH = 10000


def add_filter(component, patterns, max_match=MAX_MATCH):
    """
    Add a filter or list of filters to a component. When the component is
    a datasource, the filter will be directly added to that datasouce.
    In cases when the component is a parser or combiner, the filter will be
    added to underlying filterable datasources by traversing dependency graph.
    A filter is a simple string, and it matches if it is contained anywhere
    within a line.

    Args:
       component (component): The component to filter, can be datasource,
            parser or combiner.
       patterns (str, [str]): A string, list of strings, or set of strings to
            add to the datasource's filters.
       max_match (int): A int, the maximum matched lines to filter out.
            MAX_MATCH by default.
    """

    def get_dependency_datasources(comp):
        """Get (all) the first depended datasource"""
        dep_ds = set()
        if plugins.is_datasource(comp):
            dep_ds.add(comp)
            return dep_ds
        for dep in dr.get_dependencies(comp):
            dep_ds.update(get_dependency_datasources(dep))
        return dep_ds

    def none_max(a, b):
        return a if b is None else b if a is None else max(a, b)

    def max_matchs(da, db):
        return dict((k, none_max(da.get(k), db.get(k))) for k in set(da.keys()).union(db.keys()))

    def inner(comp, patterns):
        if comp in _CACHE:
            del _CACHE[comp]

        if not isinstance(patterns, (six.string_types, list, set)):
            raise TypeError("Filter patterns must be of type string, list, or set.")

        if isinstance(patterns, six.string_types):
            patterns = [patterns]

        for pat in patterns:
            if not pat:
                raise Exception("Filter patterns must not be empty.")

        patterns = dict((pt, max_match) for pt in patterns)
        # here patterns is a dict

        FILTERS[comp].update(max_matchs(FILTERS[comp], patterns))

    if max_match is None or type(max_match) is not int or max_match <= 0:
        raise Exception(
            "Invalid argument: {0}. It can only be a positive integer.".format(max_match)
        )

    if not plugins.is_datasource(component):
        deps = get_dependency_datasources(component)
        if deps:
            filterable_deps = [dep for dep in deps if dr.get_delegate(dep).filterable]
            # At least one dependency datasource be filterable
            if not filterable_deps:
                raise Exception("Filters aren't applicable to %s." % dr.get_name(component))

            for dep in filterable_deps:
                inner(dep, patterns)
    else:
        delegate = dr.get_delegate(component)

        if delegate.raw:
            raise Exception("Filters aren't applicable to raw datasources.")

        if not delegate.filterable:
            raise Exception("Filters aren't applicable to %s." % dr.get_name(component))

        inner(component, patterns)


_add_filter = add_filter


def get_filters(component, with_matches=False):
    """
    Get the set of filters for the given datasource.

    Filters added to a ``RegistryPoint`` will be applied to all datasources that
    implement it. Filters added to a datasource implementation apply only to
    that implementation.

    For example, a filter added to ``Specs.ps_auxww`` will apply to
    ``DefaultSpecs.ps_auxww``, ``InsightsArchiveSpecs.ps_auxww``,
    ``SosSpecs.ps_auxww``, etc. But a filter added to ``DefaultSpecs.ps_auxww``
    will only apply to ``DefaultSpecs.ps_auxww``. See the modules in
    ``insights.specs`` for those classes.

    Args:
        component (a datasource): The target datasource
        with_matches (boolean): Needs the max matches being returned? False by
                                default.

    Returns:
        (set or dict): when `with_matches=False`, returns the set of filters
                       defined for the datasource only.
                       when `with_matches=True`, returns filters defined for
                       the datasource with the max match count specified by
                       `add_filter`.
    """

    def inner(c, filters=None):
        filters = filters or dict()

        if hasattr(c, 'filterable') and c.filterable is False:
            return filters

        if not ENABLED:
            return filters

        if not plugins.is_datasource(c):
            return filters

        if c in FILTERS:
            filters.update(FILTERS[c])

        for d in dr.get_dependents(c):
            filters.update(inner(d, filters))

        return filters

    if not component:
        # No filters for nothing
        return dict() if with_matches else set()

    if component not in _CACHE:
        _CACHE[component] = inner(component)

    return _CACHE[component] if with_matches else set(_CACHE[component].keys())


def apply_filters(target, lines):
    """
    Applys filters to the lines of a datasource. This function is used only in
    integration tests. Filters are applied in an equivalent but more performant
    way at run time.
    """
    filters = get_filters(target)
    if filters:
        for l in lines:
            if any(f in l for f in filters):
                yield l
    else:
        for l in lines:
            yield l


_filename = ".".join(["filters", ser.__name__])
_dumps = ser.dump
_loads = ser.safe_load


def loads(string):
    """Loads the filters dictionary given a string."""
    d = _loads(string)
    for k, v in d.items():
        FILTERS[dr.get_component(k) or k] = v


def load(stream=None):
    """
    Loads filters from a stream, normally an open file. If one is
    not passed, filters are loaded from a default location within
    the project.
    """
    if stream:
        loads(stream.read())
    else:
        data = pkgutil.get_data(insights.__name__, _filename)
        return loads(data) if data else None


def dumps():
    """Returns a string representation of the sorted FILTERS dictionary."""
    d = {}
    for k, v in FILTERS.items():
        d[dr.get_name(k)] = dict(sorted(v.items()))
    return _dumps(d)


def dump(stream=None):
    """
    Dumps a string representation of `FILTERS` to a stream, normally an
    open file. If none is passed, `FILTERS` is dumped to a default location
    within the project.
    """
    if stream:
        stream.write(dumps())
    else:
        path = os.path.join(os.path.dirname(insights.__file__), _filename)
        with open(path, "w") as f:
            f.write(dumps())



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import dnf_module
from insights.parsers.dnf_module import DnfModuleList, DnfModuleInfo
from insights.tests import context_wrap

DNF_MODULE_LIST = """
Updating Subscription Management repositories.
Red Hat Enterprise Linux 8 for x86_64 - AppStream (RPMs)                  1.9 kB/s | 4.5 kB     00:02
Red Hat Enterprise Linux 8 for x86_64 - BaseOS (RPMs)                     1.7 kB/s | 4.0 kB     00:02
Red Hat Enterprise Linux 8 for x86_64 - Supplementary (RPMs)              1.6 kB/s | 3.8 kB     00:02
Red Hat Enterprise Linux 8 for x86_64 - AppStream (RPMs)
Name                Stream      Profiles                                  Summary
389-ds              1.4                                                   389 Directory Server (base)
ant                 1.10 [d]    common [d]                                Java build tool
container-tools     1.0         common [d]                                Common tools and dependencies for container runtimes
container-tools     rhel8 [d]   common [d]                                Common tools and dependencies for container runtimes
freeradius          3.0 [d]     server [d]                                High-performance and highly configurable free RADIUS server
gimp                2.8 [d]     common [d], devel                         gimp module
go-toolset          rhel8 [d]   common [d]                                Go
httpd               2.4 [d][e]  common [d] [i], devel, minimal            Apache HTTP Server
idm                 DL1         common [d], adtrust, client, dns, server  The Red Hat Enterprise Linux Identity Management system module
pki-core            10.6                                                  PKI Core
pki-deps            10.6                                                  PKI Dependencies module
postgresql          10 [d]      client, server [d]                        PostgreSQL server and client module
postgresql          9.6         client, server [d]                        PostgreSQL server and client module
python27            2.7 [d]     common [d]                                Python programming language, version 2.7
python36            3.6 [d][e]  common [d], build                         Python programming language, version 3.6
redis               5 [d]       common [d]                                Redis persistent key-value database
rhn-tools           1.0 [d]     common [d]                                Red Hat Satellite 5 tools for RHEL
ruby                2.5 [d]     common [d]                                An interpreter of object-oriented scripting language
ruby                2.6 [e]     common [d]                                An interpreter of object-oriented scripting language
ruby                2.7         common [d]                                An interpreter of object-oriented scripting language
rust-toolset        rhel8 [d]   common [d]                                Rust
satellite-5-client  1.0 [d]     common [d], gui                           Red Hat Satellite 5 client packages
scala               2.10 [d]    common [d]                                A hybrid functional/object-oriented language for the JVM
squid               4 [d]       common [d]                                Squid - Optimising Web Delivery
subversion          1.10 [d]    common [d], server                        Apache Subversion
swig                3.0 [d]     common [d], complete                      Connects C/C++/Objective C to some high-level programming languages
varnish             6 [d]       common [d]                                Varnish HTTP cache
virt                rhel [d]    common [d]                                Virtualization module
default_disabled    1.0 [d][x]                                            Default module which is disabled

Hint: [d]efault, [e]nabled, [x]disabled, [i]nstalled
""".strip()

DNF_MODULE_LIST_MULTI_SECTIONS = """
Last metadata expiration check: 1:37:02 ago on Wed Mar 26 09:50:23 2025.
RHEL-9.x-optional-latest
Name       Stream   Profiles                              Summary
mariadb    10.11    client, galera, server [d]            MariaDB Module
maven      3.8      common [d]                            Java project management and project comprehension tool
nginx      1.22 [e] common [d]                            nginx webserver
nginx      1.24     common [d]                            nginx webserver
redis      7        common [d]                            Redis persistent key-value database
ruby       3.1      common [d]                            An interpreter of object-oriented scripting language
ruby       3.3      common [d]                            An interpreter of object-oriented scripting language

Red Hat Enterprise Linux 9 for x86_64 - AppStream (RPMs)
Name       Stream   Profiles                              Summary
mariadb    10.11    client, galera, server [d]            MariaDB Module
maven      3.8      common [d]                            Java project management and project comprehension tool
nginx      1.22 [e] common [d]                            nginx webserver
nginx      1.24     common [d]                            nginx webserver
redis      7        common [d]                            Redis persistent key-value database
ruby       3.1      common [d]                            An interpreter of object-oriented scripting language
ruby       3.3      common [d]                            An interpreter of object-oriented scripting language
ruby       3.5      common [d]                            An interpreter of object-oriented scripting language

Hint: [d]efault, [e]nabled, [x]disabled, [i]nstalled
""".strip()

# For "Default" stream in RHEL9, take nginx module as an example, no "default"
# stream for nginx. And with no stram be enabled, it would install nginx
# version 2:1.20.1 from appstream.
DNF_MODULE_LIST_RHEL9_WO_ENABLED = """
Last metadata expiration check: 18:15:32 ago on Mon May 19 15:20:54 2025.
Red Hat Enterprise Linux 9 for x86_64 - AppStream (RPMs)
Name       Stream Profiles                                              Summary
mariadb    10.11  client, galera, server [d]                            MariaDB Module
maven      3.8    common [d]                                            Java project management and project comprehension tool
maven      3.9    common [d], openjdk11, openjdk17, openjdk21, openjdk8 Java project management and project comprehension tool
mysql      8.4    api, client, filter, server [d]                       MySQL Module
nginx      1.22   common [d]                                            nginx webserver
nginx      1.24   common [d]                                            nginx webserver
nginx      1.26   common [d]                                            nginx webserver
nodejs     18     common [d], development, minimal, s2i                 Javascript runtime
nodejs     20     common [d], development, minimal, s2i                 Javascript runtime
nodejs     22     common [d], development, minimal, s2i                 Javascript runtime
php        8.1    common [d], devel, minimal                            PHP scripting language
php        8.2    common [d], devel, minimal                            PHP scripting language
php        8.3    common [d], devel, minimal                            PHP scripting language
postgresql 15     client, server [d]                                    PostgreSQL server and client module
postgresql 16     client, server [d]                                    PostgreSQL server and client module
redis      7      common [d]                                            Redis persistent key-value database
ruby       3.1    common [d]                                            An interpreter of object-oriented scripting language
ruby       3.3    common [d]                                            An interpreter of object-oriented scripting language

Hint: [d]efault, [e]nabled, [x]disabled, [i]nstalled
""".strip()

DNF_MODULE_LIST_RHEL8_WO_ENABLED = """
Last metadata expiration check: 0:18:14 ago on Tue May 20 15:42:40 2025.
Red Hat Enterprise Linux 8 for x86_64 - AppStream (RPMs)
Name                 Stream          Profiles                                 Summary
nginx                1.14 [d]        common [d]                               nginx webserver
nginx                1.16            common [d]                               nginx webserver
nginx                1.18            common [d]                               nginx webserver
nginx                1.20            common [d]                               nginx webserver
nginx                1.22            common [d]                               nginx webserver
nginx                1.24            common [d]                               nginx webserver

Hint: [d]efault, [e]nabled, [x]disabled, [i]nstalled
""".strip()

DNF_MODULE_LIST_DOC = """
Updating Subscription Management repositories.
Name                Stream      Profiles                                  Summary
389-ds              1.4                                                   389 Directory Server (base)
ant                 1.10 [d]    common [d]                                Java build tool
ant                 1.20        common [d]                                Java build tool

Hint: [d]efault, [e]nabled, [x]disabled, [i]nstalled
""".strip()

DNF_MODULE_LIST_EXP1 = """
Updating Subscription Management repositories.
Cache-only enabled but no cache for 'rhel-8-for-x86_64-appstream-rpms', ignoring this repo.
Cache-only enabled but no cache for 'rhel-8-for-x86_64-baseos-rpms', ignoring this repo.
Cache-only enabled but no cache for 'rhel-8-for-x86_64-supplementary-rpms', ignoring this repo.
No matching Modules to list
""".strip()

DNF_MODULE_LIST_EXP2 = """
Updating Subscription Management repositories.
Name                Stream      Profiles                                  Summary
Error: xxx
""".strip()

DNF_MODULE_INFO = """
Updating Subscription Management repositories.
Last metadata expiration check: 0:02:09 ago on Thu 25 Jul 2019 01:32:41 PM CST.
Name             : httpd
Stream           : 2.4 [d][e][a]
Version          : 8000020190405071959
Context          : 55190bc5
Profiles         : common [d] [i], devel, minimal
Default profiles : common
Repo             : rhel-8-for-x86_64-appstream-rpms
Summary          : Apache HTTP Server
Description      : Apache httpd is a powerful, efficient, and extensible HTTP server.
Artifacts        : httpd-0:2.4.37-11.module+el8.0.0+2969+90015743.src
                 : httpd-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : httpd-debuginfo-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : httpd-debugsource-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : httpd-devel-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : httpd-filesystem-0:2.4.37-11.module+el8.0.0+2969+90015743.noarch
                 : httpd-manual-0:2.4.37-11.module+el8.0.0+2969+90015743.noarch
                 : httpd-tools-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : httpd-tools-debuginfo-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_http2-0:1.11.3-2.module+el8.0.0+2969+90015743.src
                 : mod_http2-0:1.11.3-2.module+el8.0.0+2969+90015743.x86_64
                 : mod_http2-debuginfo-0:1.11.3-2.module+el8.0.0+2969+90015743.x86_64
                 : mod_http2-debugsource-0:1.11.3-2.module+el8.0.0+2969+90015743.x86_64
                 : mod_ldap-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_ldap-debuginfo-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_md-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_md-debuginfo-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_proxy_html-1:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_proxy_html-debuginfo-1:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_session-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_session-debuginfo-0:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_ssl-1:2.4.37-11.module+el8.0.0+2969+90015743.x86_64
                 : mod_ssl-debuginfo-1:2.4.37-11.module+el8.0.0+2969+90015743.x86_64

Name             : httpd
Stream           : 2.4 [d][e][a]
Version          : 820190206142837
Context          : 9edba152
Profiles         : common [d] [i], devel, minimal
Default profiles : common
Repo             : rhel-8-for-x86_64-appstream-rpms
Summary          : Apache HTTP Server
Description      : Apache httpd is a powerful, efficient, and extensible HTTP server.
Artifacts        : httpd-0:2.4.37-10.module+el8+2764+7127e69e.x86_64
                 : httpd-devel-0:2.4.37-10.module+el8+2764+7127e69e.x86_64
                 : httpd-filesystem-0:2.4.37-10.module+el8+2764+7127e69e.noarch
                 : httpd-manual-0:2.4.37-10.module+el8+2764+7127e69e.noarch
                 : httpd-tools-0:2.4.37-10.module+el8+2764+7127e69e.x86_64
                 : mod_http2-0:1.11.3-1.module+el8+2443+605475b7.x86_64
                 : mod_ldap-0:2.4.37-10.module+el8+2764+7127e69e.x86_64
                 : mod_md-0:2.4.37-10.module+el8+2764+7127e69e.x86_64
                 : mod_proxy_html-1:2.4.37-10.module+el8+2764+7127e69e.x86_64
                 : mod_session-0:2.4.37-10.module+el8+2764+7127e69e.x86_64
                 : mod_ssl-1:2.4.37-10.module+el8+2764+7127e69e.x86_64

Name             : ant
Stream           : 1.10 [d][a]
Version          : 820181213135032
Context          : 5ea3b708
Profiles         : common [d]
Default profiles : common
Repo             : rhel-8-for-x86_64-appstream-rpms
Summary          : Java build tool
Description      : Apache Ant is a Java library and command-line tool whose mission is to drive processes described in build files as targets and extension points dependent upon each other. The main known usage of Ant is the build of Java applications. Ant supplies a number of built-in tasks allowing to compile, assemble, test and run Java applications. Ant can also be used effectively to build non Java applications, for instance C or C++ applications. More generally, Ant can be used to pilot any type of process which can be described in terms of targets and tasks.
Artifacts        : ant-0:1.10.5-1.module+el8+2438+c99a8a1e.noarch
                 : ant-lib-0:1.10.5-1.module+el8+2438+c99a8a1e.noarch

Name        : 389-ds
Stream      : 1.4
Version     : 8000020190424152135
Context     : ab753183
Repo        : rhel-8-for-x86_64-appstream-rpms
Summary     : 389 Directory Server (base)
Description : 389 Directory Server is an LDAPv3 compliant server.  The base package includes the LDAP server and command line utilities for server administration.
Artifacts   : 389-ds-base-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.src
            : 389-ds-base-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-debuginfo-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-debugsource-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-devel-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-legacy-tools-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-legacy-tools-debuginfo-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-libs-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-libs-debuginfo-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-snmp-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : 389-ds-base-snmp-debuginfo-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.x86_64
            : python3-lib389-0:1.4.0.20-10.module+el8.0.0+3096+101825d5.noarch

Name        : 389-ds
Stream      : 1.4
Version     : 820190201170147
Context     : 1fc8b219
Repo        : rhel-8-for-x86_64-appstream-rpms
Summary     : 389 Directory Server (base)
Description : 389 Directory Server is an LDAPv3 compliant server.  The base package includes the LDAP server and command line utilities for server administration.
Artifacts   : 389-ds-base-0:1.4.0.20-7.module+el8+2750+1f4079fb.x86_64
            : 389-ds-base-devel-0:1.4.0.20-7.module+el8+2750+1f4079fb.x86_64
            : 389-ds-base-legacy-tools-0:1.4.0.20-7.module+el8+2750+1f4079fb.x86_64
            : 389-ds-base-libs-0:1.4.0.20-7.module+el8+2750+1f4079fb.x86_64
            : 389-ds-base-snmp-0:1.4.0.20-7.module+el8+2750+1f4079fb.x86_64
            : python3-lib389-0:1.4.0.20-7.module+el8+2750+1f4079fb.noarch

Hint: [d]efault, [e]nabled, [x]disabled, [i]nstalled, [a]ctive]
""".strip()

DNF_MODULE_INFO_EXP = """
Updating Subscription Management repositories.
Last metadata expiration check: 0:03:24 ago on Thu 25 Jul 2019 01:32:41 PM CST.
Unable to resolve argument abc
Error: No matching Modules to list
""".strip()


def test_dnf_module_list():
    module_list = DnfModuleList(context_wrap(DNF_MODULE_LIST))
    assert 'ant' in module_list
    assert module_list['httpd'].name == 'httpd'
    assert module_list['httpd'].streams[0].stream == '2.4'
    assert module_list['httpd'].streams[0].default
    assert module_list['httpd'].streams[0].enabled
    assert module_list['httpd'].streams[0].active
    assert module_list['postgresql'].streams[0].stream == '10'
    assert not module_list['postgresql'].streams[0].enabled
    assert module_list['postgresql'].streams[0].active
    assert module_list['postgresql'].streams[0].default
    assert module_list['postgresql'].streams[1].stream == '9.6'
    assert not module_list['postgresql'].streams[1].enabled
    assert not module_list['postgresql'].streams[1].active
    assert not module_list['postgresql'].streams[1].default
    assert module_list['ruby'].streams[0].stream == '2.5'
    assert module_list['ruby'].streams[0].default
    assert not module_list['ruby'].streams[0].enabled
    assert not module_list['ruby'].streams[0].active
    assert module_list['ruby'].streams[1].stream == '2.6'
    assert module_list['ruby'].streams[1].enabled
    assert module_list['ruby'].streams[1].active
    assert not module_list['ruby'].streams[1].default
    assert module_list['ruby'].streams[2].stream == '2.7'
    assert not module_list['ruby'].streams[2].enabled
    assert not module_list['ruby'].streams[2].active
    assert not module_list['ruby'].streams[2].default
    assert module_list['default_disabled'].streams[0].stream == '1.0'
    assert module_list['default_disabled'].streams[0].default
    assert module_list['default_disabled'].streams[0].disabled
    assert not module_list['default_disabled'].streams[0].enabled
    assert not module_list['default_disabled'].streams[0].active
    assert len(module_list.modules) == 25

    module_list = DnfModuleList(context_wrap(DNF_MODULE_LIST_MULTI_SECTIONS))
    assert 'nginx' in module_list
    assert [s.stream for s in module_list["nginx"].streams if s.active] == ['1.22']
    assert len(module_list["nginx"].streams) == 2
    assert len(module_list["ruby"].streams) == 3

    module_list = DnfModuleList(context_wrap(DNF_MODULE_LIST_RHEL9_WO_ENABLED))
    assert 'nginx' in module_list
    assert [s.stream for s in module_list["nginx"].streams if s.active] == []
    assert len(module_list["nginx"].streams) == 3
    assert len(module_list["ruby"].streams) == 2

    module_list = DnfModuleList(context_wrap(DNF_MODULE_LIST_RHEL8_WO_ENABLED))
    assert 'nginx' in module_list
    assert [s.stream for s in module_list["nginx"].streams if s.active] == ['1.14']
    assert len(module_list["nginx"].streams) == 6


def test_dnf_module_list_exp():
    with pytest.raises(ValueError):
        DnfModuleList(context_wrap(DNF_MODULE_LIST_EXP1))

    with pytest.raises(SkipComponent):
        DnfModuleList(context_wrap(DNF_MODULE_LIST_EXP2))


def test_dnf_module_info():
    module_infos = DnfModuleInfo(context_wrap(DNF_MODULE_INFO))
    assert len(module_infos) == 3
    assert module_infos.modules == ['389-ds', 'ant', 'httpd']
    assert len(module_infos['httpd']) == 2
    assert module_infos['httpd'][0].name == 'httpd'
    assert module_infos['httpd'][0].version == '8000020190405071959'
    assert len(module_infos['httpd'][0].streams[0].profiles) == 3
    assert module_infos['httpd'][0].default_profiles == 'common'
    assert module_infos['httpd'][1].streams[0].summary == 'Apache HTTP Server'
    assert module_infos['httpd'][1].context == '9edba152'
    assert (
        'mod_http2-0:1.11.3-1.module+el8+2443+605475b7.x86_64' in module_infos['httpd'][1].artifacts
    )


def test_dnf_module_info_exp():
    with pytest.raises(SkipComponent):
        DnfModuleInfo(context_wrap(DNF_MODULE_INFO_EXP))


def test_dnf_module_doc_examples():
    env = {
        'dnf_module_list': DnfModuleList(context_wrap(DNF_MODULE_LIST_DOC)),
        'dnf_module_info': DnfModuleInfo(context_wrap(DNF_MODULE_INFO)),
    }
    failed, total = doctest.testmod(dnf_module, globs=env)
    assert failed == 0



from insights.client.connection import InsightsConnection
from mock.mock import Mock, patch


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_url_guess_legacy(get_proxies, init_session):
    """
    Connection should guess the right URLs if there's nothing in the config (the default)
    """
    config = Mock(base_url=None, upload_url=None, legacy_upload=True, analyze_container=False)

    connection = InsightsConnection(config)
    assert connection.base_url == 'https://cert-api.access.redhat.com/r/insights'
    assert connection.upload_url == 'https://cert-api.access.redhat.com/r/insights/uploads'


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_url_guess_platform(get_proxies, init_session):
    """
    Connection should guess the right URLs if there's nothing in the config (the default)
    """
    config = Mock(base_url=None, upload_url=None, legacy_upload=False)

    connection = InsightsConnection(config)
    # assert connection.base_url == 'https://console.redhat.com/api'
    assert connection.base_url == 'https://cert-api.access.redhat.com/r/insights/platform'
    # assert connection.upload_url == 'https://console.redhat.com/api/ingress/v1/upload'
    assert connection.upload_url == 'https://cert-api.access.redhat.com/r/insights/platform/ingress/v1/upload'


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_branch_info_url_guess_legacy(get_proxies, init_session):
    """
    Satellite branch info URL should be set properly
    """
    config = Mock(base_url='sat.test.com:443/redhat_access/r/insights', upload_url=None, legacy_upload=True, branch_info_url=None)

    connection = InsightsConnection(config)
    assert connection.branch_info_url == 'https://sat.test.com:443/redhat_access/r/insights/v1/branch_info'


@patch("insights.client.connection.InsightsConnection._init_session")
@patch("insights.client.connection.InsightsConnection.get_proxies")
def test_branch_info_url_guess_platform(get_proxies, init_session):
    """
    Satellite branch info URL should be the same on platform as on legacy
    """
    config = Mock(base_url='sat.test.com:443/redhat_access/r/insights', upload_url=None, legacy_upload=False, branch_info_url=None)

    connection = InsightsConnection(config)
    assert connection.branch_info_url == 'https://sat.test.com:443/redhat_access/r/insights/v1/branch_info'



from insights import rule, make_pass
from insights.core.filters import add_filter
from insights.parsers.ps import PsAux, PsAuxww, PsAlxwww
from insights.specs import Specs


@rule(PsAux)
def psaux_no_filter(ps_aux):
    return make_pass("FAKE RESULT")


add_filter(Specs.ps_auxww, "fake-filter")


@rule(PsAuxww)
def psauxww_ds_filter(ps_auxww):
    return make_pass("FAKE RESULT")


add_filter(PsAlxwww, "fake-filter")


@rule(PsAlxwww)
def psalxwww_parser_filter(ps_alxwww):
    return make_pass("FAKE RESULT")



"""
Configuration File Permissions parsers
======================================

Parsers included in this module are:

SshdConfigPerms - command ``/bin/ls -lH /etc/ssh/sshd_config``
--------------------------------------------------------------

Grub1ConfigPerms - command ``/bin/ls -lH /boot/grub/grub.conf``
---------------------------------------------------------------

Grub2ConfigPerms - command ``/bin/ls -lH /boot/grub2/grub.cfg``
---------------------------------------------------------------
"""

from insights.core import CommandParser
from insights.core.plugins import parser
from insights.specs import Specs
from insights.util.file_permissions import FilePermissions
from insights.util import deprecated


class FilePermissionsParser(CommandParser, FilePermissions):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.ls.LSlHFiles` instead.

    Base class for ``SshdConfigPerms``, ``Grub1ConfigPerms`` and ``Grub2ConfigPerms`` classes.

    Attributes:
        line (string): the line from the command output
    """

    def __init__(self, context):
        deprecated(
            FilePermissionsParser, "Please use insights.parsers.ls.LSlHFiles instead.", "3.7.0"
        )
        self.line = ""
        CommandParser.__init__(self, context)
        FilePermissions.__init__(self, self.line)

    def parse_content(self, content):
        non_empty_lines = [line for line in content if line]  # get rid of blank lines
        self.line = non_empty_lines[0]


@parser(Specs.sshd_config_perms)
class SshdConfigPerms(FilePermissionsParser):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.ls.LSlHFiles` instead.

    Class for parsing ``/bin/ls -lH /etc/ssh/sshd_config`` command.

    Sample output of this command is::

        -rw-------. 1 root root 4179 Dec  1  2014 /etc/ssh/sshd_config

    Examples:
        >>> type(sshd_perms)
        <class 'insights.parsers.config_file_perms.SshdConfigPerms'>
        >>> sshd_perms.line
        '-rw-------. 1 root root 4179 Dec  1  2014 /etc/ssh/sshd_config'
    """

    def __init__(self, context):
        super(SshdConfigPerms, self).__init__(context)


@parser(Specs.grub1_config_perms)
class Grub1ConfigPerms(FilePermissionsParser):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.ls.LSlHFiles` instead.

    Class for parsing ``/bin/ls -lH /boot/grub/grub.conf`` command.

    Sample output of this command is::

        -rw-r--r--. 1 root root 4179 Dec  1  2014 /boot/grub/grub.conf

    Examples:
        >>> type(grub1_perms)
        <class 'insights.parsers.config_file_perms.Grub1ConfigPerms'>
        >>> grub1_perms.line
        '-rw-r--r--. 1 root root 4179 Dec  1  2014 /boot/grub/grub.conf'
    """

    def __init__(self, context):
        super(Grub1ConfigPerms, self).__init__(context)


@parser(Specs.grub_config_perms)
class Grub2ConfigPerms(FilePermissionsParser):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.ls.LSlHFiles` instead.

    Class for parsing ``/bin/ls -lH /boot/grub2/grub.cfg`` command.

    Sample output of this command is::

        -rw-r--r--. 1 root root 4179 Dec  1  2014 /boot/grub2/grub.cfg

    Examples:
        >>> type(grub2_perms)
        <class 'insights.parsers.config_file_perms.Grub2ConfigPerms'>
        >>> grub2_perms.line
        '-rw-r--r--. 1 root root 4179 Dec  1  2014 /boot/grub2/grub.cfg'
    """

    def __init__(self, context):
        super(Grub2ConfigPerms, self).__init__(context)



import doctest
import pytest

from insights.tests import context_wrap
from insights.parsers import machine_id
from insights.core.exceptions import SkipComponent

MACHINE_ID_CONTENT = """dc194312-8cdd-4e75-8cf1-2094bf666f45 hostname1,hostname2"""

WRONG_CONTENT_1 = ""

WRONG_CONTENT_2 = """
dc194312-8cdd-4e75-8cf1-2094bf666f25 hostname1,hostname2
dc194312-8cdd-4e75-8cf1-2094bf666asd hostname3,hostname4
""".strip()

WRONG_CONTENT_3 = """
dc194312-8cdd-4e75-8cf1-2094bf666f25
""".strip()


def test_duplicate_id():
    machine_obj = machine_id.DuplicateMachine(context_wrap(MACHINE_ID_CONTENT))
    assert machine_obj.duplicate_id == 'dc194312-8cdd-4e75-8cf1-2094bf666f45'
    assert machine_obj.hostnames == ['hostname1', 'hostname2']


def test_exception():
    with pytest.raises(SkipComponent):
        machine_id.DuplicateMachine(context_wrap(WRONG_CONTENT_1))

    with pytest.raises(SkipComponent):
        machine_id.DuplicateMachine(context_wrap(WRONG_CONTENT_2))

    with pytest.raises(SkipComponent):
        machine_id.DuplicateMachine(context_wrap(WRONG_CONTENT_3))


def test_doc():
    machine_obj = machine_id.DuplicateMachine(context_wrap(MACHINE_ID_CONTENT))
    env = {'machine_id_obj': machine_obj}
    failed, total = doctest.testmod(machine_id, globs=env)
    assert failed == 0



"""
Sap
===
This combiner combines the following parsers for SAP instances:
:class:`insights.parsers.saphostctrl.SAPHostCtrlInstances` and
:class:`insights.parsers.hostname.Hostname`
"""
from collections import namedtuple

from insights import SkipComponent
from insights.combiners.hostname import Hostname
from insights.core.plugins import combiner
from insights.parsers.saphostctrl import SAPHostCtrlInstances

SAPInstances = namedtuple(
    "SAPInstances",
    field_names=["name", "hostname", "sid", "type", "full_type", "number", "fqdn", "version"])
"""namedtuple: Type for storing the SAP instance."""

DAA_TYPES = {
    'Solution Manager Diagnostic Agent': ['SMDA'],
    'Diagnostic Agent': ['DAA', 'SMD'],
}
NETW_TYPES = ('D', 'ASCS', 'DVEBMGS', 'J', 'SCS', 'ERS', 'W', 'G', 'JC')
"""
tuple: Tuple of the prefix string of the functional SAP instances::

    D      :    NetWeaver (ABAP Dialog Instance)
    ASCS   :    NetWeaver (ABAP Central Services)
    DVEBMGS:    NetWeaver (Primary Application server)
    J      :    NetWeaver (Java App Server Instance)
    SCS    :    NetWeaver (Java Central Services)
    ERS    :    NetWeaver (Enqueue Replication Server)
    W      :    NetWeaver (WebDispatcher)
    G      :    NetWeaver (Gateway)
    JC     :    NetWeaver (Java App Server Instance)
"""


@combiner(Hostname, SAPHostCtrlInstances)
class Sap(dict):
    """
    It combines the following parsers for SAP instances:
    :class:`insights.parsers.saphostctrl.SAPHostCtrlInstances` and
    :class:`insights.parsers.hostname.Hostname`

    Attributes:
    Examples:
        >>> type(saps)
        <class 'insights.combiners.sap.Sap'>
        >>> 'DVEBMGS12' in saps
        True
        >>> saps['DVEBMGS12'].number
        '12'
        >>> saps.sid('ASCS10')
        'R4D'
        >>> saps.hostname('ASCS10')
        'host_1'
        >>> len(saps.instances)
        3
        >>> saps.is_hana
        False
        >>> saps.is_netweaver
        True
        >>> saps.is_ascs
        True
    """
    def __init__(self, hostname, insts):
        hn = hostname.hostname
        fqdn = hostname.fqdn
        self._local_instances = []
        self._instances = []
        self._daa_instances = []

        self._types = insts.types
        self._all_instances = insts.instances
        for inst in insts:
            k = inst['InstanceName']
            if (hn == inst['Hostname'].split('.')[0] or
                    fqdn == inst['FullQualifiedHostname'] or
                    fqdn == inst['Hostname']):
                self._local_instances.append(k)
            self[k] = SAPInstances(k,
                                   inst['Hostname'],
                                   inst['SID'],
                                   inst['InstanceName'].strip('1234567890'),
                                   inst['InstanceType'],
                                   inst['SystemNumber'],
                                   inst['FullQualifiedHostname'],
                                       inst['SapVersionInfo'])
        if len(self) == 0:
            raise SkipComponent('No SAP instance.')

        for i in self.values():
            if (
                i.full_type in DAA_TYPES or
                i.type in [st for sts in DAA_TYPES.values() for st in sts]
            ):
                self._daa_instances.append(i.name)
            else:
                self._instances.append(i.name)

    def version(self, instance):
        """str: Returns the version of the ``instance``."""
        return self[instance].version if instance in self else None

    def sid(self, instance):
        """str: Returns the sid of the ``instance``."""
        return self[instance].sid if instance in self else None

    def type(self, instance):
        """str: Returns the short type code of the ``instance``."""
        return self[instance].type if instance in self else None

    def full_type(self, instance):
        """str: Returns the full type code of the ``instance``."""
        return self[instance].full_type if instance in self else None

    def hostname(self, instance):
        """str: Returns the hostname of the ``instance``."""
        return self[instance].hostname if instance in self else None

    def number(self, instance):
        """str: Returns the systeme number of the ``instance``."""
        return self[instance].number if instance in self else None

    @property
    def is_netweaver(self):
        """bool: Is any SAP NetWeaver instance detected?"""
        return any(_t in self._types for _t in NETW_TYPES)

    @property
    def is_hana(self):
        """bool: Is any SAP HANA instance detected?"""
        return 'HDB' in self._types

    @property
    def is_ascs(self):
        """bool: Is any ABAP Central Services instance detected?"""
        return 'ASCS' in self._types

    @property
    def all_instances(self):
        """
        list: List of all the SAP instances listed by the command.
        """
        return self._all_instances

    @property
    def instances(self):
        """
        list: List of the SAP instances that are NOT Diagnostic Agent
              instance, e.g. HANA, NetWeaver, ASCS, or other instances
        """
        return self._instances

    @property
    def daa_instances(self):
        """
        list: List of the SAP Diagnostic Agent instances, e.g.
              Diagnostics Agents: SMDA97/SMDA98
        """
        return self._daa_instances



"""
OpenvSwitchOtherConfig - command ``ovs-vsctl -t 5 get Open_vSwitch . other_config``
===================================================================================

Class OpenvSwitchOtherConfig process the output of the following OpenvSwitch command:

ovs-vsctl -t 5 get Open_vSwitch . other_config


Sample input::

    {dpdk-init="true", dpdk-lcore-mask="30000003000", dpdk-socket-mem="4096,4096", pmd-cpu-mask="30000003000"}

Examples:

    >>> type(ovs_other_conf)
    <class 'insights.parsers.openvswitch_other_config.OpenvSwitchOtherConfig'>
    >>> ovs_other_conf.get("dpdk-init")
    true
    >>> ovs_other_conf["dpdk-lcore-mask"]
    30000003000
    >>> dpdk-socket-mem in ovs_other_conf
    True

"""

from .. import parser, LegacyItemAccess, CommandParser
from insights.specs import Specs
from . import optlist_to_dict


@parser(Specs.openvswitch_other_config)
class OpenvSwitchOtherConfig(LegacyItemAccess, CommandParser):
    """Parses output of the ovs-vsctl -t 5 get Open_vSwitch . other_config command"""

    def parse_content(self, content):
        self.data = {}
        one_line_content = ''.join(content).strip()
        if one_line_content.startswith("{"):
            new_line = one_line_content.strip("{}")
            if new_line:
                self.data = optlist_to_dict(new_line, opt_sep=", ", strip_quotes=True)



from insights.parsers.ipcs import IpcsSI, IpcsS
from insights.parsers.ps import PsAuxcww
from insights.combiners import ipcs_semaphores
from insights.combiners.ipcs_semaphores import IpcsSemaphores
from insights.tests import context_wrap
import doctest

PsAuxcww_OUT = """
USER       PID %CPU %MEM    VSZ   RSS TTY      STAT START   TIME COMMAND
root         1  0.0  0.0  19356  1544 ?        Ss   May31   0:01 init
root      6152  0.0  0.0      0     0 ?        S    May31   0:29 kondemand/0
root      2265  0.0  0.0  18244   668 ?        Ss   May31   0:05 irqbalance
""".strip()

IPCS_S = """
------ Semaphore Arrays --------
key        semid      owner      perms      nsems
0x00000000 557056     apache     600        4
0x00000000 655371     apache     600        1
0x0052e2c1 65536      postgres   600        8
0x00000000 622502     apache     600        1
0x00000000 622602     apache     600        1
0x00000000 688140     apache     600        1
""".strip()

IPCS_S_I_1 = """
Semaphore Array semid=557056
uid=500  gid=501     cuid=500    cgid=501
mode=0600, access_perms=0600
nsems = 4
otime = Sun May 12 14:44:53 2013
ctime = Wed May  8 22:12:15 2013
semnum     value      ncount     zcount     pid
0          1          0          0          0
1          1          0          0          0
4          1          0          0          0
5          1          0          0          0
"""

IPCS_S_I_2 = """
Semaphore Array semid=65536
uid=500  gid=501     cuid=500    cgid=501
mode=0600, access_perms=0600
nsems = 8
otime = Sun May 12 14:44:53 2013
ctime = Wed May  8 22:12:15 2013
semnum     value      ncount     zcount     pid
0          1          0          0          0
1          1          0          0          0
0          1          0          0          6151
3          1          0          0          2265
4          1          0          0          0
5          1          0          0          0
0          0          7          0          6152
7          1          0          0          4390

"""

IPCS_S_I_3 = """
Semaphore Array semid=622502
uid=48   gid=48  cuid=0  cgid=0
mode=0600, access_perms=0600
nsems = 1
otime = Wed Apr 12 15:41:03 2017
ctime = Wed Apr 12 15:41:03 2017
semnum     value      ncount     zcount     pid
0          1          0          0          6151

"""

IPCS_S_I_4 = """
Semaphore Array semid=622602
uid=48   gid=48  cuid=0  cgid=0
mode=0600, access_perms=0600
nsems = 1
otime = Wed Apr 12 15:41:03 2017
ctime = Wed Apr 12 15:41:03 2017
semnum     value      ncount     zcount     pid
0          1          0          0          6151

"""

IPCS_S_I_5 = """
Semaphore Array semid=655371
uid=48   gid=48  cuid=0  cgid=0
mode=0600, access_perms=0600
nsems = 1
otime = Not set
ctime = Wed Apr 12 15:41:03 2017
semnum     value      ncount     zcount     pid
0          1          0          0          991

"""

IPCS_S_I_6 = """
Semaphore Array semid=688140
uid=48   gid=48  cuid=0  cgid=0
mode=0600, access_perms=0600
nsems = 1
otime = Wed Apr 12 16:01:28 2017
ctime = Wed Apr 12 15:41:03 2017
semnum     value      ncount     zcount     pid
0          0          7          0          6152
""".strip()


def test_ipcs_semaphores():
    sem1 = IpcsSI(context_wrap(IPCS_S_I_1))
    sem2 = IpcsSI(context_wrap(IPCS_S_I_2))
    sem3 = IpcsSI(context_wrap(IPCS_S_I_3))
    sem4 = IpcsSI(context_wrap(IPCS_S_I_4))
    sem5 = IpcsSI(context_wrap(IPCS_S_I_5))
    sem6 = IpcsSI(context_wrap(IPCS_S_I_6))
    sems = IpcsS(context_wrap(IPCS_S))
    ps = PsAuxcww(context_wrap(PsAuxcww_OUT))
    rst = IpcsSemaphores(sems, [sem1, sem2, sem3, sem4, sem5, sem6], ps)
    assert rst.get_sem('65536').pid_list == ['0', '2265', '4390', '6151', '6152']
    assert rst.count_of_all_sems() == 6
    assert rst.count_of_all_sems(owner='apache') == 5
    assert rst.count_of_orphan_sems() == 3
    assert rst.count_of_orphan_sems('postgres') == 0
    i = 0
    for sem in rst:
        i += 1
    assert i == rst.count_of_all_sems()
    assert rst.orphan_sems() == ['622502', '622602', '655371']
    assert rst.orphan_sems('apache') == ['622502', '622602', '655371']
    assert rst.orphan_sems('postgres') == []


def test_doc_examples():
    sem1 = IpcsSI(context_wrap(IPCS_S_I_1))
    sem2 = IpcsSI(context_wrap(IPCS_S_I_2))
    sem3 = IpcsSI(context_wrap(IPCS_S_I_3))
    sem4 = IpcsSI(context_wrap(IPCS_S_I_4))
    sem6 = IpcsSI(context_wrap(IPCS_S_I_6))
    sems = IpcsS(context_wrap(IPCS_S))
    ps = PsAuxcww(context_wrap(PsAuxcww_OUT))
    env = {
            'oph_sem': IpcsSemaphores(sems,
                        [sem1, sem2, sem3, sem4, sem6], ps)
          }
    failed, total = doctest.testmod(ipcs_semaphores, globs=env)
    assert failed == 0



import sys
import os
import yaml
import pkgutil

INSTALL_DIR = os.path.dirname(os.path.abspath(__file__))
NAME = "insights.yaml"
DEFAULTS_NAME = "defaults.yaml"


def load_and_read(path):
    if os.path.exists(path):
        with open(path) as fp:
            return fp.read()


CONFIGS = [
    pkgutil.get_data('insights', 'defaults.yaml'),
    load_and_read(os.path.join("/etc", NAME)),  # System-wide config
    load_and_read(os.path.join(os.path.expanduser("~/.local"), NAME)),  # User-specific config
    load_and_read("." + NAME)  # Directory-specific config
]

config = {}

for c in CONFIGS:
    if c is None:
        continue
    try:
        y = yaml.safe_load(c)
        for name, section in y.items():
            if name in config:
                config[name].update(section)
            else:
                config[name] = section
    except Exception as e:
        print(c)
        print(e)

# The defaults section is for keys that belong in every section and can be
# overridden in particular sections if desired.  This adds the default values
# to each section if they aren't already there.

for k in config["defaults"]:
    for section in set(s for s in config if s != "defaults"):
        if k not in config[section]:
            config[section][k] = config["defaults"][k]

# Flatten the attribute hierarchy a bit by cutting out "config".
for name, section in config.items():
    setattr(sys.modules[__name__], name, section)



"""
ContainersPolicy - file ``/etc/containers/policy.json``
=======================================================
"""
from insights import JSONParser, parser
from insights.specs import Specs


@parser(Specs.containers_policy)
class ContainersPolicy(JSONParser):
    """
    Class for converting file ``/etc/containers/policy.json``
    into a dictionary that matches the JSON string in the file.

    Sample file content::

        {
          "default": [
            {
              "type": "insecureAcceptAnything"
            }
          ],
          "transports": {
            "docker": {
              "registry.access.redhat.com": [
                {
                  "type": "signedBy",
                  "keyType": "GPGKeys",
                  "keyPath": "/etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release"
                }
              ],
              "registry.redhat.io/redhat/redhat-operator-index": [
                {
                  "type": "insecureAcceptAnything"
                }
              ],
              "registry.redhat.io": [
                {
                  "type": "signedBy",
                  "keyType": "GPGKeys",
                  "keyPath": "/etc/pki/rpm-gpg/RPM-GPG-KEY-redhat-release"
                }
              ]
            },
            "docker-daemon": {
              "": [
                {
                  "type": "insecureAcceptAnything"
                }
              ]
            }
          }
        }

    Examples:
        >>> len(containers_policy["default"])
        1
    """
    pass



from insights.parsers.dmidecode import DMIDecode
from insights.tests import context_wrap

from datetime import date

DMIDECODE = '''
# dmidecode 2.11
SMBIOS 2.7 present.
188 structures occupying 5463 bytes.
Table at 0xBFFCB000.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: HP
\tVersion: P70
\tRelease Date: 03/01/2013
\tAddress: 0xF0000
\tRuntime Size: 64 kB
\tROM Size: 8192 kB
\tCharacteristics:
\t\tPCI is supported
\t\tPNP is supported
\t\tBIOS is upgradeable
\t\tBIOS shadowing is allowed
\t\tESCD support is available
\t\tBoot from CD is supported
\t\tSelectable boot is supported
\t\tEDD is supported
\t\t5.25"/360 kB floppy services are supported (int 13h)
\t\t5.25"/1.2 MB floppy services are supported (int 13h)
\t\t3.5"/720 kB floppy services are supported (int 13h)
\t\tPrint screen service is supported (int 5h)
\t\tj042 keyboard services are supported (int 9h)
\t\tSerial services are supported (int 14h)
\t\tPrinter services are supported (int 17h)
\t\tCGA/mono video services are supported (int 10h)
\t\tACPI is supported
\t\tUSB legacy is supported
\t\tBIOS boot specification is supported
\t\tFunction key-initiated network boot is supported
\t\tTargeted content distribution is supported
\tFirmware Revision: 1.22

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: HP
\tProduct Name: ProLiant DL380p Gen8
\tVersion: Not Specified
\tSerial Number: 2M23360006
\tUUID: 34373936-3439-4D32-3233-333630303036
\tWake-up Type: Power Switch
\tSKU Number: 697494-S01
\tFamily: ProLiant

Handle 0x0300, DMI type 3, 21 bytes
chassis information
\tmanufacturer: hp
\ttype: rack mount chassis
\tlock: not present
\tversion: not specified
\tserial number: 2m23360006
\tasset Tag:
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
\tOEM Information: 0x00000000
\tHeight: 2 U
\tNumber Of Power Cords: 2
\tContained Elements: 0

Handle 0x0401, DMI type 4, 32 bytes
Processor Information
\tSocket Designation: CPU 1
\tType: Central Processor
\tFamily: Other
\tManufacturer: Bochs
\tID: A1 06 02 00 FD FB 8B 17
\tVersion: Not Specified
\tVoltage: Unknown
\tExternal Clock: Unknown
\tMax Speed: 2000 MHz
\tCurrent Speed: 2000 MHz
\tStatus: Populated, Enabled
\tUpgrade: Other
\tL1 Cache Handle: Not Provided
\tL2 Cache Handle: Not Provided
\tL3 Cache Handle: Not Provided

Handle 0x0402, DMI type 4, 32 bytes
Processor Information
\tSocket Designation: CPU 2
\tType: Central Processor
\tFamily: Other
\tManufacturer: Bochs
\tID: A1 06 02 00 FD FB 8B 17
\tVersion: Not Specified
\tVoltage: Unknown
\tExternal Clock: Unknown
\tMax Speed: 2000 MHz
\tCurrent Speed: 2000 MHz
\tStatus: Populated, Enabled
\tUpgrade: Other
\tL1 Cache Handle: Not Provided
\tL2 Cache Handle: Not Provided
\tL3 Cache Handle: Not Provided

Handle 0x0037, DMI type 127, 4 bytes.
End Of Table
'''

DMIDECODE_V = '''
# dmidecode 2.12
SMBIOS 2.4 present.
364 structures occupying 16870 bytes.
Table at 0x000E0010.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: Phoenix Technologies LTD
\tVersion: 6.00
\tRelease Date: 04/14/2014
\tAddress: 0xEA050
\tRuntime Size: 90032 bytes
\tROM Size: 64 kB
\tCharacteristics:
\t\tISA is supported
\t\tPCI is supported
\t\tPC Card (PCMCIA) is supported
\t\tPNP is supported
\t\tAPM is supported
\t\tBIOS is upgradeable
\t\tBIOS shadowing is allowed
\t\tESCD support is available
\t\tBoot from CD is supported
\t\tSelectable boot is supported
\t\tEDD is supported
\t\tPrint screen service is supported (int 5h)
\t\t8042 keyboard services are supported (int 9h)
\t\tSerial services are supported (int 14h)
\t\tPrinter services are supported (int 17h)
\t\tCGA/mono video services are supported (int 10h)
\t\tACPI is supported
\t\tSmart battery is supported
\t\tBIOS boot specification is supported
\t\tFunction key-initiated network boot is supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 4.6
\tFirmware Revision: 0.0

Handle 0x0001, DMI type 1, 27 bytes
System Information
\tManufacturer: VMware, Inc.
\tProduct Name: VMware Virtual Platform
\tVersion: None
\tSerial Number: VMware-42 10 e9 13 1e 11 e2 21-13 c5 c8 1f 11 42 7c cb
\tUUID: 4210E913-1E11-E221-13C5-C81F11427CCB
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Not Specified
'''

DMIDECODE_AWS = '''
# dmidecode 2.12-dmifs
SMBIOS 2.4 present.
11 structures occupying 310 bytes.
Table at 0x000EB01F.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: Xen
\tVersion: 4.2.amazon
\tRelease Date: 12/09/2016
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tPCI is supported
\t\tEDD is supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 4.2

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: Xen
\tProduct Name: HVM domU
\tVersion: 4.2.amazon
\tSerial Number: ec2f58af-2dad-c57e-88c0-a81cb6084290
\tUUID: EC2F58AF-2DAD-C57E-88C0-A81CB6084290
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Not Specified

Handle 0x0300, DMI type 3, 13 bytes
Chassis Information
\tManufacturer: Xen
\tType: Other
\tLock: Not Present
\tVersion: Not Specified
\tSerial Number: Not Specified
\tAsset Tag: Not Specified
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
'''

DMIDECODE_KVM = '''
# dmidecode 3.0
Scanning /dev/mem for entry point.
SMBIOS 2.8 present.
13 structures occupying 788 bytes.
Table at 0xBFFFFCE0.

Handle 0x0000, DMI type 0, 24 bytes
BIOS Information
\tVendor: SeaBIOS
\tVersion: 1.9.1-5.el7_3.2
\tRelease Date: 04/01/2014
\tAddress: 0xE8000
\tRuntime Size: 96 kB
\tROM Size: 64 kB
\tCharacteristics:
\t\tBIOS characteristics not supported
\t\tTargeted content distribution is supported
\tBIOS Revision: 0.0

Handle 0x0100, DMI type 1, 27 bytes
System Information
\tManufacturer: Red Hat
\tProduct Name: RHEV Hypervisor
\tVersion: 7.3-7.el7
\tSerial Number: 34353737-3035-4E43-3734-353130425732
\tUUID: 10331906-BB22-4716-8876-3DFEF8FA941E
\tWake-up Type: Power Switch
\tSKU Number: Not Specified
\tFamily: Red Hat Enterprise Linux

Handle 0x0300, DMI type 3, 21 bytes
Chassis Information
\tManufacturer: Red Hat
\tType: Other
\tLock: Not Present
\tVersion: RHEL 7.2.0 PC (i440FX + PIIX, 1996)
\tSerial Number: Not Specified
\tAsset Tag: Not Specified
\tBoot-up State: Safe
\tPower Supply State: Safe
\tThermal State: Safe
\tSecurity Status: Unknown
\tOEM Information: 0x00000000
\tHeight: Unspecified
\tNumber Of Power Cords: Unspecified
\tContained Elements: 0
'''

DMIDECODE_FAIL = "# dmidecode 2.11\n# No SMBIOS nor DMI entry point found, sorry.\n"

DMIDECODE_DMI = '''
# dmidecode 2.2
SMBIOS 2.4 present.
104 structures occupying 3162 bytes.
Table at 0x000EE000.
Handle 0x0000
\tDMI type 0, 24 bytes.
\tBIOS Information
\t\tVendor: HP
\t\tVersion: A08
\t\tRelease Date: 09/27/2008
\t\tAddress: 0xF0000
\t\tRuntime Size: 64 kB
\t\tROM Size: 4096 kB
\t\tCharacteristics:
\t\t\tPCI is supported
\t\t\tPNP is supported
\t\t\tBIOS is upgradeable
\t\t\tBIOS shadowing is allowed
\t\t\tESCD support is available
\t\t\tBoot from CD is supported
\t\t\tSelectable boot is supported
\t\t\tEDD is supported
\t\t\t5.25"/360 KB floppy services are supported (int 13h)
\t\t\t5.25"/1.2 MB floppy services are supported (int 13h)
\t\t\t3.5"/720 KB floppy services are supported (int 13h)
\t\t\tPrint screen service is supported (int 5h)
\t\t\t8042 keyboard services are supported (int 9h)
\t\t\tSerial services are supported (int 14h)
\t\t\tPrinter services are supported (int 17h)
\t\t\tCGA/mono video services are supported (int 10h)
\t\t\tACPI is supported
\t\t\tUSB legacy is supported
\t\t\tBIOS boot specification is supported
\t\t\tFunction key-initiated network boot is supported``
Handle 0x0100
\tDMI type 1, 27 bytes.
\tSystem Information
\t\tManufacturer: HP
\t\tProduct Name: ProLiant BL685c G1
\t\tVersion: Not Specified
\t\tSerial Number: 3H6CMK2537
\t\tUUID: 58585858-5858-3348-3643-4D4B32353337
\t\tWake-up Type: Power Switch
'''

# Several oddities in processing that should be picked up:
# * Things besides 'Characteristics' that are double-indented
# * Multiple instances of the same heading
# * A subheading with a value on that line and subsequent double-indented data.
DMIDECODE_ODDITIES = """
Handle 0x0009, DMI type 129, 8 bytes
OEM-specific Type
\tHeader and Data:
\t\t81 08 09 00 01 01 02 01
\tStrings:
\t\tIntel_ASF
\t\tIntel_ASF_001

Handle 0x000A, DMI type 134, 13 bytes
OEM-specific Type
\tHeader and Data:
\t\t86 0D 0A 00 28 06 14 20 00 00 00 00 00

Handle 0x000F, DMI type 8, 9 bytes
Port Connector Information
\tInternal Reference Designator: Not Available
\tInternal Connector Type: None
\tExternal Reference Designator: USB 1
\tExternal Connector Type: Access Bus (USB)
\tPort Type: USB

Handle 0x0010, DMI type 8, 9 bytes
Port Connector Information
\tInternal Reference Designator: Not Available
\tInternal Connector Type: None
\tExternal Reference Designator: USB 2
\tExternal Connector Type: Access Bus (USB)
\tPort Type: USB

Handle 0x0029, DMI type 13, 22 bytes
BIOS Language Information
\tLanguage Description Format: Abbreviated
\tInstallable Languages: 1
\t\ten-US
\tCurrently Installed Language: en-US

Handle 0x000A, DMI type 134, 13 bytes
OEM-specific Type
\tHeader and Data:
\t\t01 02 03 04 05 06 07 08

"""


def test_get_dmidecode():
    '''
    Test for three kinds of output format of dmidecode parser
    '''
    context = context_wrap(DMIDECODE)
    ret = DMIDecode(context)

    assert len(ret.get("bios_information")) == 1
    assert ret.get("bios_information")[0].get("vendor") == "HP"
    assert ret.get("bios_information")[0].get("version") == "P70"
    assert ret.get("bios_information")[0].get("release_date") == "03/01/2013"
    assert ret.get("bios_information")[0].get("address") == "0xF0000"
    assert ret.get("bios_information")[0].get("runtime_size") == "64 kB"
    assert ret.get("bios_information")[0].get("rom_size") == "8192 kB"

    tmp = ["PCI is supported", "PNP is supported", "BIOS is upgradeable",
           "BIOS shadowing is allowed", "ESCD support is available",
           "Boot from CD is supported", "Selectable boot is supported",
           "EDD is supported",
           '5.25"/360 kB floppy services are supported (int 13h)',
           '5.25"/1.2 MB floppy services are supported (int 13h)',
           '3.5"/720 kB floppy services are supported (int 13h)',
           "Print screen service is supported (int 5h)",
           "j042 keyboard services are supported (int 9h)",
           "Serial services are supported (int 14h)",
           "Printer services are supported (int 17h)",
           "CGA/mono video services are supported (int 10h)",
           "ACPI is supported", "USB legacy is supported",
           "BIOS boot specification is supported",
           "Function key-initiated network boot is supported",
           "Targeted content distribution is supported"]
    assert ret.get("bios_information")[0].get("characteristics") == tmp
    assert ret.get("bios_information")[0].get("firmware_revision") == "1.22"

    assert len(ret.get("system_information")) == 1
    assert ret.get("system_information")[0].get("manufacturer") == "HP"
    assert ret.get("system_information")[0].get("product_name") == "ProLiant DL380p Gen8"
    assert ret.get("system_information")[0].get("version") == "Not Specified"
    assert ret.get("system_information")[0].get("serial_number") == "2M23360006"
    assert ret.get("system_information")[0].get("uuid") == "34373936-3439-4D32-3233-333630303036"
    assert ret.get("system_information")[0].get("wake-up_type") == "Power Switch"
    assert ret.get("system_information")[0].get("sku_number") == "697494-S01"
    assert ret.get("system_information")[0].get("family") == "ProLiant"

    assert len(ret.get("chassis_information")) == 1
    assert ret.get("chassis_information")[0].get("manufacturer") == "hp"
    assert ret.get("chassis_information")[0].get("type") == "rack mount chassis"
    assert ret.get("chassis_information")[0].get("lock") == "not present"
    assert ret.get("chassis_information")[0].get("version") == "not specified"
    assert ret.get("chassis_information")[0].get("serial_number") == "2m23360006"
    assert ret.get("chassis_information")[0].get("asset_tag") == ""
    assert ret.get("chassis_information")[0].get("boot-up_state") == "Safe"
    assert ret.get("chassis_information")[0].get("power_supply_state") == "Safe"
    assert ret.get("chassis_information")[0].get("thermal_state") == "Safe"
    assert ret.get("chassis_information")[0].get("security_status") == "Unknown"
    assert ret.get("chassis_information")[0].get("oem_information") == "0x00000000"
    assert ret.get("chassis_information")[0].get("height") == "2 U"
    assert ret.get("chassis_information")[0].get("number_of_power_cords") == "2"
    assert ret.get("chassis_information")[0].get("manufacturer") == "hp"
    assert ret.get("chassis_information")[0].get("contained_elements") == "0"

    assert len(ret.get('processor_information')) == 2
    assert ret.get("processor_information")[0].get("socket_designation") == "CPU 1"
    assert ret.get("processor_information")[0].get("type") == "Central Processor"

    assert ret.get("processor_information")[1].get("socket_designation") == "CPU 2"
    assert ret.get("processor_information")[1].get("type") == "Central Processor"

    # Check for 'nonsense' keys
    assert 'table_at_0xbffcb000.' not in ret

    # Test property accessors
    assert ret.system_info == ret['system_information'][0]
    assert ret.system_uuid == '34373936-3439-4D32-3233-333630303036'.lower()
    assert ret.bios == ret['bios_information'][0]
    assert ret.bios_vendor == 'HP'
    assert ret.bios_date == date(2013, 3, 1)
    assert ret.processor_manufacturer == 'Bochs'


def test_get_dmidecode_fail():
    '''
    Test for faied raw data
    '''
    context = context_wrap(DMIDECODE_FAIL)
    ret = DMIDecode(context)

    assert ret.is_present is False
    assert ret.system_uuid is None


# def test_virt_what_1():
#     '''
#     Test for virt_what()
#     '''
#     context = context_wrap(DMIDECODE_V)
#     ret = DMIDecode(context)
#     assert ret.is_present is True
#     assert ret.virt_what == "vmware"


# def test_virt_what_2():
#     '''
#     Test for virt_what() with AWS data
#     '''
#     context = context_wrap(DMIDECODE_AWS)
#     ret = DMIDecode(context)
#     assert ret.is_present is True
#     assert ret.virt_what == "amazon"


# def test_virt_what_3():
#     '''
#     Test for virt_what() with KVM
#     '''
#     context = context_wrap(DMIDECODE_KVM)
#     ret = DMIDecode(context)
#     assert ret.is_present is True
#     assert ret.virt_what == "kvm"


def test_get_dmidecode_dmi():
    '''
    Test for three kinds of output format of dmidecode parser
    with special input format:
    "\n\tDMI" in the input
    '''
    context = context_wrap(DMIDECODE_DMI)
    ret = DMIDecode(context)

    assert ret.get("bios_information")[0].get("vendor") == "HP"
    assert ret.get("bios_information")[0].get("version") == "A08"
    assert ret.get("bios_information")[0].get("release_date") == "09/27/2008"
    assert ret.get("bios_information")[0].get("address") == "0xF0000"
    assert ret.get("bios_information")[0].get("runtime_size") == "64 kB"
    assert ret.get("bios_information")[0].get("rom_size") == "4096 kB"

    tmp = ["PCI is supported", "PNP is supported", "BIOS is upgradeable",
           "BIOS shadowing is allowed", "ESCD support is available",
           "Boot from CD is supported", "Selectable boot is supported",
           "EDD is supported",
           '5.25"/360 KB floppy services are supported (int 13h)',
           '5.25"/1.2 MB floppy services are supported (int 13h)',
           '3.5"/720 KB floppy services are supported (int 13h)',
           "Print screen service is supported (int 5h)",
           "8042 keyboard services are supported (int 9h)",
           "Serial services are supported (int 14h)",
           "Printer services are supported (int 17h)",
           "CGA/mono video services are supported (int 10h)",
           "ACPI is supported", "USB legacy is supported",
           "BIOS boot specification is supported",
           "Function key-initiated network boot is supported``"]
    assert ret.get("bios_information")[0].get("characteristics") == tmp

    assert ret.get("system_information")[0].get("manufacturer") == "HP"
    assert ret.get("system_information")[0].get("product_name") == "ProLiant BL685c G1"
    assert ret.get("system_information")[0].get("version") == "Not Specified"
    assert ret.get("system_information")[0].get("serial_number") == "3H6CMK2537"
    assert ret.get("system_information")[0].get("uuid") == "58585858-5858-3348-3643-4D4B32353337"
    assert ret.get("system_information")[0].get("wake-up_type") == "Power Switch"
    assert ret.system_uuid == '58585858-5858-3348-3643-4D4B32353337'.lower()


def test_dmidecode_oddities():
    dmi = DMIDecode(context_wrap(DMIDECODE_ODDITIES))

    assert len(dmi['oem-specific_type']) == 3
    assert dmi['oem-specific_type'][0] == {
        'header_and_data': '81 08 09 00 01 01 02 01',
        'strings': ['Intel_ASF', 'Intel_ASF_001'],
    }
    assert dmi['oem-specific_type'][1] == {
        'header_and_data': '86 0D 0A 00 28 06 14 20 00 00 00 00 00',
    }
    assert dmi['oem-specific_type'][2] == {
        'header_and_data': '01 02 03 04 05 06 07 08',
    }

    assert len(dmi['port_connector_information']) == 2
    assert dmi['port_connector_information'][0] == {
        'internal_reference_designator': 'Not Available',
        'internal_connector_type': 'None',
        'external_reference_designator': 'USB 1',
        'external_connector_type': 'Access Bus (USB)',
        'port_type': 'USB',
    }
    assert dmi['port_connector_information'][0] == {
        'internal_reference_designator': 'Not Available',
        'internal_connector_type': 'None',
        'external_reference_designator': 'USB 1',
        'external_connector_type': 'Access Bus (USB)',
        'port_type': 'USB',
    }

    assert len(dmi['bios_language_information']) == 1
    assert dmi['bios_language_information'][0] == {
        'language_description_format': 'Abbreviated',
        'installable_languages': ['1', 'en-US'],
        'currently_installed_language': 'en-US'
    }



import doctest
import pytest

from insights.tests import context_wrap
from insights.core.dr import SkipComponent
from insights.parsers import lsattr


LSATTR_OUTPUT1 = """
./grub2-tools-2.02-0.86.el7.x86_64.rpm
""".strip()

LSATTR_OUTPUT2 = """
---------------- ./grub2-tools-2.02-0.86.el7.x86_64.rpm

------i--------- ./grub2-common-2.02-0.86.el7.noarch.rpm
---------------- ./grub2-tools-minimal-2.02-0.86.el7.x86_64.rpm
lsattr: No such file or directory while trying to stat a/f
"""

LSATTR_OUTPUT3 = """


"""

LSATTR_OUTPUT4 = """
lsattr: No such file or directory while trying to stat a/f
"""

LSATTR_OUTPUT5 = """
---------------- ./grub2-tools-2.02-0.86.el7.x86_64.rpm
------i--------- ./grub2-common-2.02-0.86.el7.noarch.rpm
---------------- ./grub2-tools-minimal-2.02-0.86.el7.x86_64.rpm
"""


def test_doc():
    failed_count, _ = doctest.testmod(
        lsattr,
        globs={'lsattr_obj': lsattr.LsAttr(context_wrap(LSATTR_OUTPUT2))}
    )
    assert failed_count == 0


def test_except():
    with pytest.raises(SkipComponent):
        lsattr.LsAttr(context_wrap(LSATTR_OUTPUT1))
    with pytest.raises(SkipComponent):
        lsattr.LsAttr(context_wrap(LSATTR_OUTPUT3))


def test_normal_output():
    attr_obj = lsattr.LsAttr(context_wrap(LSATTR_OUTPUT2))
    assert len(attr_obj) == 4
    assert attr_obj['./grub2-tools-2.02-0.86.el7.x86_64.rpm'] == ''
    assert attr_obj['./grub2-common-2.02-0.86.el7.noarch.rpm'] == 'i'
    assert attr_obj['./grub2-tools-minimal-2.02-0.86.el7.x86_64.rpm'] == ''
    assert 'non-exist' in attr_obj
    assert len(attr_obj['non-exist']) == 1
    assert attr_obj['non-exist'][0] == 'lsattr: No such file or directory while trying to stat a/f'

    attr_obj2 = lsattr.LsAttr(context_wrap(LSATTR_OUTPUT4))
    assert 'non-exist' in attr_obj2
    assert len(attr_obj2) == 1
    assert len(attr_obj2['non-exist']) == 1
    assert attr_obj2['non-exist'][0] == 'lsattr: No such file or directory while trying to stat a/f'

    attr_obj3 = lsattr.LsAttr(context_wrap(LSATTR_OUTPUT5))
    assert 'non-exist' not in attr_obj3



import pytest

try:
    from unittest.mock import patch
except Exception:
    from mock import patch

from insights.client.config import InsightsConfig


@pytest.mark.parametrize(
    ("obfuscate", "obfuscate_hostname", "obfuscation_list", "expected_opt"),
    [
        (None, None, None, None),
        (False, False, None, []),
        ('False', 'False', None, []),
        ('0', '0', None, []),
        ('NO', 'No', None, []),
        (None, None, '', []),
        (True, True, None, ['ipv4', 'hostname']),
        ('True', 'True', None, ['ipv4', 'hostname']),
        ('YES', 'yes', None, ['ipv4', 'hostname']),
        (True, False, None, ['ipv4']),
        ('true', 'FALSE', None, ['ipv4']),
        ('1', '0', None, ['ipv4']),
        ('Yes', 'no', None, ['ipv4']),
        (None, None, 'ipv6, hostname, ipv4', ['hostname', 'ipv4', 'ipv6']),
        (None, None, 'ipv4,ipv6,hostname,mac', ['hostname', 'ipv4', 'ipv6', 'mac']),
    ],
)
def test_validate_obfuscation_options_good(
    obfuscate, obfuscate_hostname, obfuscation_list, expected_opt
):
    c = InsightsConfig(
        obfuscate=obfuscate,
        obfuscate_hostname=obfuscate_hostname,
        obfuscation_list=obfuscation_list,
    )
    assert c.obfuscation_list == expected_opt
    assert c.obfuscate is None
    assert c.obfuscate_hostname is None


@patch('insights.client.config.logger')
@patch('insights.client.config.get_rhel_version', return_value=10)
@patch('insights.client.config.get_egg_version_tuple', return_value=(3, 6, 10))
def test_validate_obfuscation_options_conflict_old_warning(egg, rhel, logger):
    with pytest.raises(ValueError) as ve:
        InsightsConfig(obfuscate=False, obfuscate_hostname=True, _print_errors=True)
    assert 'Option `obfuscate_hostname` requires `obfuscate`' in str(ve.value)
    logger.warning.assert_called_once_with(
        'WARNING: `obfuscate` and `obfuscate_hostname` are deprecated, please use `obfuscation_list` instead.'
    )


@patch('insights.client.config.logger')
@patch('insights.client.config.get_rhel_version', return_value=10)
@patch('insights.client.config.get_egg_version_tuple', return_value=(3, 5, 10))
def test_validate_obfuscation_options_conflict_old_no_warning(egg, rhel, logger):
    InsightsConfig(obfuscate=True, obfuscate_hostname=True, _print_errors=True)
    logger.warning.assert_not_called()


@patch('insights.client.config.logger')
@patch('insights.client.config.get_rhel_version', return_value=7)
@patch('insights.client.config.get_egg_version_tuple', return_value=(3, 6, 12))
def test_validate_obfuscation_options_conflict_old_rhel7(egg, rhel, logger):
    with pytest.raises(ValueError) as ve:
        InsightsConfig(obfuscate=False, obfuscate_hostname=True, _print_errors=True)
    assert 'Option `obfuscate_hostname` requires `obfuscate`' in str(ve.value)
    logger.warning.assert_not_called()


@pytest.mark.parametrize(
    ("obfuscate", "obfuscate_hostname", "obfuscation_list", "expected_opt"),
    [
        (True, True, '', []),
        (True, False, '', []),
        (False, False, '', []),
        (True, True, 'ipv6, hostname, ipv4', ['hostname', 'ipv4', 'ipv6']),
        (False, False, 'ipv4,ipv6,hostname,mac', ['hostname', 'ipv4', 'ipv6', 'mac']),
    ],
)
@patch('insights.client.config.logger')
@patch('insights.client.config.get_rhel_version', return_value=10)
def test_validate_obfuscation_options_conflict_new(
    rhel, logger, obfuscate, obfuscate_hostname, obfuscation_list, expected_opt
):
    c = InsightsConfig(
        obfuscate=obfuscate,
        obfuscate_hostname=obfuscate_hostname,
        obfuscation_list=obfuscation_list,
        _print_errors=True,
    )
    assert c.obfuscation_list == expected_opt
    logger.warning.assert_called_once_with(
        'WARNING: Conflicting options: `obfuscation_list` and `obfuscate`, using: "obfuscation_list={0}".'.format(
            obfuscation_list
        )
    )


@patch('insights.client.config.logger')
def test_validate_obfuscation_options_new_invalid(logger):
    c = InsightsConfig(obfuscation_list='mac, abc, ipv4', _print_errors=True)
    assert c.obfuscation_list == ['ipv4', 'mac']
    logger.warning.assert_called_once_with(
        'WARNING: ignoring invalid obfuscate options: `abc`, using: "obfuscation_list=ipv4,mac".'
    )



"""
RHSM Release Version - ``file /var/lib/rhsm/cache/releasever.json``
===================================================================
Parser Red Hat Subscription manager release info.

"""
from insights.core import JSONParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.rhsm_releasever)
class RhsmReleaseVer(JSONParser):
    """
    Class for parsing the file: ``/var/lib/rhsm/cache/releasever.json``.

    This information mirror the information provided by the
    ``subscription-manager release --show`` command.

    .. note::
        Please refer to the super-class :class:`insights.core.JSONParser`
        for additional information on attributes and methods.

    Sample input data::

        {"releaseVer": "6.10"}

    Raises:
       SkipComponent: When the json content of the file is empty.(i.e release version is empty. eg. {})

    Examples:
        >>> type(rhsm_releasever)
        <class 'insights.parsers.rhsm_releasever.RhsmReleaseVer'>
        >>> rhsm_releasever['releaseVer'] == '6.10'
        True
        >>> rhsm_releasever.set == '6.10'
        True
        >>> rhsm_releasever.major
        6
        >>> rhsm_releasever.minor
        10
    """

    def parse_content(self, content):
        """
        Parse the contents of file ``/var/lib/rhsm/cache/releasever.json``.
        """
        super(RhsmReleaseVer, self).parse_content(content)
        self.set = self.major = self.minor = None
        if 'releaseVer' not in self.data:
            raise SkipComponent('releaseVer is not in data')
        rel = self.data.get('releaseVer') or ''
        rel_splits = rel.split('.')
        # Release: 6.7
        if len(rel_splits) == 2:
            if rel_splits[0].isdigit() and rel_splits[-1].isdigit():
                self.set = rel
                self.major = int(rel_splits[0])
                self.minor = int(rel_splits[-1])

        # Release: 7Server or 8
        elif rel and rel[0].isdigit():
            self.set = rel
            self.major = int(rel[0])
            # leave self.minor as None



import six
from pytest import mark

try:
    import ipaddress
except Exception:
    from insights.contrib import ipaddress

from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner


IPv6_cases = """
a FE80::2E21:3101:34FB:F580 dev enp3s0 2c:21:31:fb:f5:80 router
X ff02::2 dev enp3s0 lladdr 33:33:00:00:00:02 NOARP
b ff02::16 dev enp3s0 lladdr 33:33:00:00:00:16 NOARP
3 fe80::2204:fff:fe83:7678/64 lladdr 20:04:0f:83:76:78 router
2620:52:0:4996::1fe dev enp3s0 lladdr 2c:21:31:fb:f5:80 STALE
ff02::1:ff47:17 dev enp3s0 lladdr 33:33:ff:47:00:17 NOARP
::1 dev lo lladdr 00:00:00:00:00:00 NOARP
fe80::2204:fff:fe76:224b dev enp3s0 lladdr router STALE - line 1
dev enp3s0 lladdr router 9d8a::5ef5:342:ac2a:ab52
D ::1/32 NOARP
e ::/32 NOARP
F ::ffff:/32 NOARP

fe80::2204:fff:fe76:224b dev enp3s0 lladdr router STALE - line 2
""".strip().splitlines()

IPv6_cases_keep_same = """
fe80::2204:fff:fe76:224b dev enp3s0 lladdr router STALE - line 1
FE80::2204:FFF:FE76:224B dev enp3s0 lladdr router STALE - line 2
dev enp3s0 lladdr router fe80:0000:0000:0000:2204:fff:fe76:224b
dev enp3s0 lladdr router fe80:0:0:0:2204:fff:fe76:224b
""".strip().splitlines()


@mark.parametrize("obfuscation_list", [[], ['ipv6']])
def test_obfuscate_ipv6(obfuscation_list):
    c = InsightsConfig(obfuscation_list=obfuscation_list)
    pp = Cleaner(c, {})
    ip_changed = 0
    len_changed = 0
    ret0 = pp.clean_content(IPv6_cases)
    for idx, line in enumerate(ret0):
        org_line = IPv6_cases[idx]
        if line != org_line:
            ip_changed += 1
        if len(line) != len(org_line):
            len_changed += 1
    if obfuscation_list:
        assert ip_changed == len(IPv6_cases) - 5  # empty and 4 lines of "::" are not obfuscated
        assert len_changed == 0  # length of obfuscated IPv6 is not changed
    else:
        assert ip_changed == 0
        assert len_changed == 0
    # No obfuscated IP address will be obfuscated:
    ret1 = pp.clean_content(ret0)
    assert ret1 == ret0


@mark.parametrize("obfuscation_list", [[], ['ipv6']])
def test_obfuscate_ipv6_the_same(obfuscation_list):
    c = InsightsConfig(obfuscation_list=obfuscation_list)
    pp = Cleaner(c, {})
    ip_changed = 0
    len_changed = 0
    ret0 = pp.clean_content(IPv6_cases_keep_same)
    for idx, line in enumerate(ret0):
        org_line = IPv6_cases_keep_same[idx]
        if line != org_line:
            ip_changed += 1
        if len(line) != len(org_line):
            len_changed += 1
    if obfuscation_list:
        assert len_changed == 0  # length of obfuscated IPv6 is not changed
        assert ip_changed == len(IPv6_cases_keep_same)
        # Four IPs are obfuscated
        obf_ips = pp.obfuscate.get('ipv6').mapping()
        assert len(obf_ips) == 4
        # But they are all the same - 1: in result
        ret_ip = ipaddress.ip_address(
            ret0[0].split()[0] if six.PY3 else u'{0}'.format(ret0[0].split()[0])
        )
        assert ret_ip == ipaddress.ip_address(
            ret0[1].split()[0] if six.PY3 else u'{0}'.format(ret0[1].split()[0])
        )
        assert ret_ip == ipaddress.ip_address(
            ret0[2].split()[-1] if six.PY3 else u'{0}'.format(ret0[2].split()[-1])
        )
        assert ret_ip == ipaddress.ip_address(
            ret0[3].split()[-1] if six.PY3 else u'{0}'.format(ret0[3].split()[-1])
        )
        # But they are all the same - 2: in mapping
        for ip in obf_ips:
            assert (
                ipaddress.ip_address(
                    ip['obfuscated'] if six.PY3 else u'{0}'.format(ip['obfuscated'])
                )
                == ret_ip
            )
    else:
        assert ip_changed == 0
        assert len_changed == 0
    # No obfuscated IP address will be obfuscated:
    ret1 = pp.clean_content(ret0)
    assert ret1 == ret0


def test_obfuscate_ipv6_no_obfuscate():
    c = InsightsConfig(obfuscation_list=['ipv6'])
    pp = Cleaner(c, {})
    # no_obfuscate=['ipv6'] will not obfuscate any IPv6 address
    ret = pp.clean_content(IPv6_cases, no_obfuscate=['ipv6', 'mac'])
    assert ret == IPv6_cases
    # no_obfuscate=['ipv4'] will only obfuscate IPv4 address but not IPv6 address
    pp = Cleaner(c, {})  # Must initialize a new instance
    ret = pp.clean_content(IPv6_cases, no_obfuscate=['ipv4', 'mac'])
    assert ret != IPv6_cases



import pytest
from mock.mock import Mock, patch

from insights.core.exceptions import SkipComponent
from insights.specs.datasources.mdadm import raid_devices

MOCK_OS_STAT_ST_MODE = {
    "b": 25008,
    "d": 16877,
    "-": 33188,
}

GLOB_DEV_MD_1 = "/dev/md1 /dev/md2 /dev/md3".split()
GLOB_DEV_MD_2 = "/dev/md".split()
GLOB_DEV_MD_3 = "/dev/mdxyz".split()
GLOB_DEV_MD_4 = "".split()

mock_os_stat_return_1 = Mock(st_mode=MOCK_OS_STAT_ST_MODE["b"])
mock_os_stat_return_2 = Mock(st_mode=MOCK_OS_STAT_ST_MODE["d"])
mock_os_stat_return_3 = Mock(st_mode=MOCK_OS_STAT_ST_MODE["-"])


@patch("os.stat", return_value=mock_os_stat_return_1)
@patch("glob.glob", return_value=GLOB_DEV_MD_1)
def test_raid_devices(m_glob, m_stst):
    result = raid_devices({})
    assert result is not None
    assert isinstance(result, str)
    assert result == "/dev/md1 /dev/md2 /dev/md3"


@patch("os.stat", return_value=mock_os_stat_return_2)
@patch("glob.glob", return_value=GLOB_DEV_MD_2)
def test_raid_devices_bad_2(m_glob, m_stst):
    with pytest.raises(SkipComponent) as e:
        raid_devices({})
    assert "Skipping raid_devices: No /dev/md* raid devices found" in str(e)


@patch("os.stat", return_value=mock_os_stat_return_3)
@patch("glob.glob", return_value=GLOB_DEV_MD_3)
def test_raid_devices_bad_3(m_glob, m_stst):
    with pytest.raises(SkipComponent) as e:
        raid_devices({})
    assert "Skipping raid_devices: No /dev/md* raid devices found" in str(e)


@patch("os.stat", return_value=mock_os_stat_return_1)
@patch("glob.glob", return_value=GLOB_DEV_MD_4)
def test_raid_devices_bad_4(m_glob, m_stst):
    with pytest.raises(SkipComponent) as e:
        raid_devices({})
    assert "Skipping raid_devices: No /dev/md* raid devices found" in str(e)



import datetime
import pytest

from insights.core import YAMLParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.tests import context_wrap


bi_conf_content = """
{"remote_branch": -1, "remote_leaf": -1}
""".strip()

yaml_test_strings = {
    """
type:        Acquisition
date:        2019-07-09
""": {
        'type': 'Acquisition',
        'date': datetime.date(2019, 7, 9),
    },
    """
- Hesperiidae
- Papilionidae
- Apatelodidae
- Epiplemidae
""": [
        'Hesperiidae',
        'Papilionidae',
        'Apatelodidae',
        'Epiplemidae',
    ],
}

yaml_content_with_extra_lines = """
test=abc
---
server:
  hostname: abc
""".strip()

empty_yaml_content = """
---
# This YAML file is empty
""".strip()

wrong_yaml_content = """
"unbalanced blackets: ]["
""".strip()


class FakeYamlParser(YAMLParser):
    """Class for parsing the content of ``branch_info``."""

    pass


class MyYamlParser(YAMLParser):
    ignore_lines = ['test=']


def test_yaml_parser_success():
    for ymlstr in yaml_test_strings:
        ctx = context_wrap(ymlstr)
        assert FakeYamlParser(ctx).data == yaml_test_strings[ymlstr]


def test_yaml_parser_success_skip_lines():
    ctx = context_wrap(yaml_content_with_extra_lines)
    assert MyYamlParser(ctx).get('server').get('hostname') == 'abc'


def test_yaml_parser_failure():
    ctx = context_wrap("boom /")
    with pytest.raises(ParseException) as ex:
        FakeYamlParser(ctx)

    assert "FakeYamlParser" in ex.value.args[0]


def test_settings_yml():
    ctx = context_wrap(bi_conf_content)
    ctx.content = bi_conf_content
    result = FakeYamlParser(ctx)
    assert result.data['remote_branch'] == -1
    assert result.data['remote_leaf'] == -1


def test_settings_yml_list():
    ctx = context_wrap(bi_conf_content)
    result = FakeYamlParser(ctx)
    assert result.data['remote_branch'] == -1
    assert result.data['remote_leaf'] == -1


def test_empty_content():
    ctx = context_wrap(empty_yaml_content)
    with pytest.raises(SkipComponent) as ex:
        FakeYamlParser(ctx)

    assert "There is no data" in ex.value.args[0]


def test_yaml_parser_with_equal_value():
    ctx = context_wrap("key: =")
    assert FakeYamlParser(ctx).data == {"key": "="}

    ctx = context_wrap("key: value")
    assert FakeYamlParser(ctx).data == {"key": "value"}



import os
import os.path
import pytest

from insights.core.exceptions import ContentException
from insights.core.spec_factory import FileProvider

SAMPLE_ROOT = "sample/root"
SAMPLE_FILE = "sample_file.txt"


@pytest.fixture(scope="module")
def sample_directory(tmpdir_factory):
    # str() required for Python 2.7
    root = str(tmpdir_factory.mktemp("test_file_provider"))

    def create_directory(relpath):
        dir_path = os.path.join(root, relpath)
        # Python 2.7 does not support exist_ok and raises an exception when path exists
        try:
            os.makedirs(dir_path)
        except:
            pass

    def create_file(relpath, content=""):
        file_path = os.path.join(root, relpath)
        create_directory(os.path.dirname(file_path))
        fd = open(file_path, "w")
        fd.write(content)
        fd.close()

    create_file(SAMPLE_FILE)
    create_directory("sample/root")
    create_file("sample/root/access_denied.txt")
    os.chmod(os.path.join(root, "sample/root/access_denied.txt"), 0)
    return root


class DummyFileProvider(FileProvider):
    def load():
        return ""


def test_repr(sample_directory):
    provider = DummyFileProvider(SAMPLE_FILE, root=sample_directory)
    # example: DummyFileProvider('/tmp/pytest0/test_file_provider/sample_file.txt')
    assert repr(provider) == "DummyFileProvider('%s/%s')" % (sample_directory, SAMPLE_FILE)


@pytest.mark.skipif(
    # GitHub workflows run Python 2.7 and 3.6 tests in containers, because these old Python
    # versions are not available in GitHub-hosted runners anymore. The tests are executed as root.
    os.getuid() == 0,
    reason="Test must not be run as root. Root ignores file mode bits."
)
@pytest.mark.parametrize(
    "relpath,exception_type,match",
    [
        ("access_denied.txt", ContentException, "Cannot access"),
        ("no_such_file.txt", ContentException, "does not exist"),
        # file outside root that exists
        ("../../sample_file.txt", Exception, "Relative path points outside the root"),
    ]
)
def test_validate(sample_directory, relpath, exception_type, match):
    with pytest.raises(exception_type, match=match):
        DummyFileProvider(relpath, root=os.path.join(sample_directory, SAMPLE_ROOT))
